********************************************************************************
*                                                                              *
*  2DHF version 2-2006                                                         *
*  Copyright (C) 1996  Jacek Kobus, Leif Laaksonen, Dage Sundholm              *
*                                                                              *
*  This software may be used and distributed according to the terms            *
*  of the GNU General Public License, see README and COPYING.                  *
*                                                                              *
********************************************************************************
c ### difmu ###

c     This routine calculates 

c     (\frac{\partial^2}{\partial\mu^2} + 
c           b(\ni,\mu) \frac{\partial}{\partial \mu}) f(\ni,\mu)

c     Function f has been imersed in the array f(nni+8,nmu+8) in order
c     to calculated derivatives in all the grid points.  Originally the
c     routine was used for a single grid of constatnt step size
c     hmu. Accordingly dmu array contained the first- and second-order
c     derivative coefficients (taken from the 8th-order Sterling
c     interpolation formula) multiplied by the b array.

c     To make the routine work in the multigrid case (ngrids.ne.1) 
c     the values of dmu(k,imu) for 

c     imu=iemu(1)-3 ... iemu(1)+3
c     imu=iemu(2)-3 ... iemu(2)+3
c      .
c     imu=iemu(ngrids-1)-3 ... iemu(ngrids-1)+3

c     must be prepared with derivative coefficients which are based on
c     other interpolation formulae taking into account different grid
c     density to the left and right of the grid boundaries. See prepfix
c     for detailes.

c     This routine calculates 

c     {\partial^2 / \partial\mu^2 + 
c           b(\ni,\mu) \partial / \partial \mu} f(\ni,\mu)

c     Function f has been imersed in the array f(nni+8,nmu+8) in order
c     to calculated derivatives in all the grid points.
c     Originally the routine was used for a single grid of constatnt step
c     size hmu. Accordingly dmu array contained the first- and second-order
c     derivative coefficients (taken from the 8th-order Sterling
c     interpolation formula) multiplied by the B array. 

c     To make the routine work in the multigrid case (ngrids.ne.1!) 
c     the values of dmu(k,imu) for 

c     imu=iemu(1)-3 ... iemu(1)+3
c     imu=iemu(2)-3 ... iemu(2)+3
c     .
c     imu=iemu(ngrids-1)-3 ... iemu(ngrids-1)+3

c     must be prepared with derivative coefficients which are based on
c     other interpolation formula taking into account different grid
c     density to the left and right of the grid boundaries. See prepfix
c     for detailes.

c     $Id: difmu.f,v 1.4 2006/06/28 20:43:48 jkob Exp $	

      subroutine difmu (n,f,fd)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      include 'commons8.inc'
      character*1 trans
      data trans/'n'/,incr/1/,alpha/1.d0/,beta/0.d0/
      data n9/9/
      dimension f(nni+8,*),fd(nni+8,*)	

c     The following loop runs now till j=mxnmu so that the derivatives are
c     also defined in the tail region, i.e. for (i,mxnmu-3),...,(i,mxnmu)
c     To this end the fill routine provides extra values for
c     (i,mxnmu+1),...,(i,mxnmu+4) points.


      nni8=nni+8
      do j=1,n
c         call mxv(f(1,j),nni8,dmu(1,j),n9,fd(1,j+4))
         call dgemv (trans,nni8,n9,alpha,f(1,j),nni8,dmu(1,j),incr,
     &        beta,fd(1,j+4),incr)
      enddo
      return
      end



      subroutine dif1mu (n,f,fd)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      include 'commons8.inc'
      character*1 trans
      data trans/'n'/,incr/1/,alpha/1.d0/,beta/0.d0/
      data n9/9/
      dimension f(nni+8,*),fd(nni+8,*)	

c     The following loop runs now till j=mxnmu so that the derivatives are
c     also defined in the tail region, i.e. for (i,mxnmu-3),...,(i,mxnmu)
c     To this end the fill routine provides extra values for
c     (i,mxnmu+1),...,(i,mxnmu+4) points.

      nni8=nni+8
      do j=1,n
c         call mxv(f(1,j),nni8,d1mu(1,j),n9,fd(1,j+4))
         call dgemv (trans,nni8,n9,alpha,f(1,j),nni8,dmu(1,j),incr,
     &        beta,fd(1,j+4),incr)
      enddo
      return
      end
