********************************************************************************
*                                                                              * 
*  2DHF version 2-2006                                                         *
*  Copyright (C) 2006 Jacek Kobus                                              *
*                                                                              *
*  This software may be used and distributed according to the terms            *
*  of the GNU General Public License, see README and COPYING.                  *
*                                                                              *
********************************************************************************
c ### eclyptot ###

c     Calculates correlation energy according to Lee, Yang, Prr (PRB 37
c     (1988) 785-789)

      real*8 function eclyptot (psi,wgt2,rhot,rhotup,rhotdown,
     &     grhot,grhotup,grhotdown,wk0,wk1,wk2,wk3,wk4,wk5,wk6,wk7)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      parameter (const13=1.d0/3.d0,const83=8.d0/3.d0,const23=2.d0/3.d0,
     &     const53=5.d0/3.d0,const19=1.d0/9.d0,const118=1.d0/18.d0)

c     coefficients of the Colle-Salvetti formula 
      parameter(a=0.04918d0,b=0.132d0,c=0.2533d0,d=0.349d0)

      include 'commons8.inc'	

      dimension psi(*),wgt2(*),wk0(*),wk1(*),wk2(*),wk3(*),wk4(*),  
     &     wk5(*),wk6(*),wk7(*),  
     &     rhot(*),rhotup(*),rhotdown(*),
     &     grhot(*),grhotup(*),grhotdown(*)

      cf=0.3d0*(three*pii*pii)**const23

      do i=1,mxsize
         rhotup(i)  =0.d0
         rhotdown(i)=0.d0
      enddo

c     calculate total densities for up and down spins
      do iorb=1,norb
         iborb=i1b(iorb)
         isiorb=i1si(iorb)
         nmut=i1mu(iorb)
         
         call exocc (iorb,ocup,ocdown)

         call prod2 (isiorb,psi(iborb),psi(iborb),wk1)
         call dscal (isiorb,ocup,wk1,1)
         
         call prod2 (isiorb,psi(iborb),psi(iborb),wk2)
         call dscal (isiorb,ocdown,wk2,1)
       
c        store total spin densities 
         call add(isiorb,wk1,rhotup)
         call add(isiorb,wk2,rhotdown)
      enddo

c     calculate nabla rho nabla rho (up)
      call nfng (rhotup,rhotup,wk0,wk1,wk2,wk3,wk4,wk5,wk6,grhotup)
      
c     calculate nabla rho nabla rho (down)
      call nfng (rhotdown,rhotdown,wk0,wk1,wk2,wk3,wk4,wk5,wk6,
     &     grhotdown)


c     calculate nabla^2 rho (up)
      call n2f (rhotup,wk0,wk1,wk2,rhot)

c     calculate nabla^2 rho (down)
      call n2f (rhotdown,wk0,wk1,wk2,grhot)

c     tw (up)
      do i=1,mxsize
         if (abs(rhotup(i)).lt.precis) then
            wk1(i)=0.d0
         else
            wk1(i)=(grhotup(i)/rhotup(i)-rhot(i))/8.d0
         endif
      enddo

c     tw (down)
      do i=1,mxsize
         if (abs(rhotdown(i)).lt.precis) then
            wk2(i)=0.d0
         else
            wk2(i)=(grhotdown(i)/rhotdown(i)-grhot(i))/8.d0
         endif
      enddo

c     wk7 - total density, wk6 - tw(total)
      do i=1,mxsize
         wk7(i)=rhotup(i)+rhotdown(i)
         if (abs(wk7(i)).lt.precis) then
c FIXME
            wk6(i)=two
         else
            wk6(i)=two*
     &           (one-
     &                (rhotup(i)*rhotup(i)+rhotdown(i)*rhotdown(i))/
     &                 (wk7(i)*wk7(i))
     &           )
         endif
      enddo	  

c     wk4 - Eq.22 terms from square brackets except for rhot tw
      do i=1,mxsize
         wk4(i)=two**const23*cf*
     &        (rhotup(i)**const83+rhotdown(i)**const83) 
     &        +const19*(rhotup(i)*wk1(i)+rhotdown(i)*wk2(i))
     &        +const118*(rhotup(i)*rhot(i)+rhotdown(i)*grhot(i))
      enddo

c     wk7->rhot
      call dcopy(mxsize,wk7,ione,rhot,ione)

c     wk4->grhotup
      call dcopy(mxsize,wk4,ione,grhot,ione)

c     calculate nabla rho nabla rho (up+down)
      call nfng (rhot,rhot,wk0,wk1,wk2,wk3,wk4,wk5,wk6,grhotup)

c     calculate nabla^2 rho (up+down)
      call n2f (rhot,wk0,wk1,wk2,grhotdown)

c     wk5 - tw
c     wk6 - gamma (total) 
      do i=1,mxsize
         if (rhot(i).lt.precis) then
c FIXME
            wk5(i)=0.d0
            wk6(i)=two
         else
            wk5(i)=(grhotup(i)/rhot(i)-grhotdown(i))/8.d0
            wk6(i)=two*
     &           (one-
     &                (rhotup(i)*rhotup(i)+rhotdown(i)*rhotdown(i))/
     &                 (rhot(i)*rhot(i))
     &           )
         endif
      enddo	  

      do i=1,mxsize
         if (rhot(i).lt.precis) then
            wk0(i)=0.d0
         else
c     wk4/grhot +  rhot tw
            t1=grhot(i)-rhot(i)*wk5(i)
            t2=two*b*rhot(i)**(-const53)*t1*exp(-c*rhot(i)**(-const13))
            wk0(i)=-a*(rhot(i)+t2)*wk6(i)/(one+d*rhot(i)**(-const13))
         endif

      enddo

c     take care of f4 factor
      call multf4(wk0)

      eclyptot=ddot(mxsize,wgt2,ione,wk0,ione)

      return
      end

