********************************************************************************
*                                                                              * 
*  2DHF version 2-2006                                                         *
*  Copyright (C) 2006 Jacek Kobus                                              *
*                                                                              *
*  This software may be used and distributed according to the terms            *
*  of the GNU General Public License, see README and COPYING.                  *
*                                                                              *
********************************************************************************
c ### eclyptot ###

c     Calculates exchange correlation energy according to Vosko, Wilk, Nusair
c     Can. J. Phys. 58 (1980) 1200

      real*8 function ecvwntot (psi,wgt2,rhot,rhotup,rhotdown,
     &     grhot,grhotup,grhotdown,wk0,wk1,wk2,wk3,wk4,wk5,wk6,wk7)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      parameter (const13=1.d0/3.d0,const16=1.d0/6.d0)

      parameter(ck1=0.0310907d0,cl1=-0.10498d0,
     &     cm1=3.72744d0,cn1=12.9352d0)

      include 'commons8.inc'	

      dimension psi(*),wgt2(*),wk0(*),wk1(*),wk2(*),wk3(*),wk4(*),  
     &     wk5(*),wk6(*),wk7(*),  
     &     rhot(*),rhotup(*),rhotdown(*),
     &     grhot(*),grhotup(*),grhotdown(*)

      constx=(three/(four*pii))**const16

      do i=1,mxsize
         rhotup(i)  =0.d0
         rhotdown(i)=0.d0
      enddo

c     calculate total densities for up and down spins
      do iorb=1,norb
         iborb=i1b(iorb)
         isiorb=i1si(iorb)
         nmut=i1mu(iorb)
         
         call exocc (iorb,ocup,ocdown)

         call prod2 (isiorb,psi(iborb),psi(iborb),wk1)
         call dscal (isiorb,ocup,wk1,1)
         
         call prod2 (isiorb,psi(iborb),psi(iborb),wk2)
         call dscal (isiorb,ocdown,wk2,1)
       
c        store total spin densities 
         call add(isiorb,wk1,rhotup)
         call add(isiorb,wk2,rhotdown)
      enddo


      do i=1,mxsize
         rhot(i)=rhotup(i)+rhotdown(i)
      enddo	  

      do i=1,mxsize
         if (rhot(i).lt.precis) then
            wk0(i)=0.d0
         else
            x=constx*rhot(i)**(-const16)

            wk0(i)=rhot(i)*qvwn(x,ck1,cl1,cm1,cn1)
         endif

      enddo

c     take care of f4 factor
      call multf4(wk0)

      ecvwntot=ddot(mxsize,wgt2,ione,wk0,ione)

      return
      end
