********************************************************************************
*                                                                              * 
*  2DHF version 2-2006                                                         *
*  Copyright (C) 2006 Jacek Kobus                                              *
*                                                                              *
*  This software may be used and distributed according to the terms            *
*  of the GNU General Public License, see README and COPYING.                  *
*                                                                              *
********************************************************************************
c ### exbe88 ###

c     Calculates exchange energy according to Becke eq.8 (PRA 38
c     (1988) 3098-3100)

c     $Id: exbe88.f,v 1.4 2006/06/28 09:56:28 jkob Exp $	

      real*8 function exbe88 (psi,wgt2,rhot,rhotup,rhotdown,
     &     grhot,grhotup,grhotdown,wk0,wk1,wk2,wk3,wk4,wk5,wk6,wk7)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      parameter (const13=1.d0/3.d0,const43=4.d0/3.d0,
     &     const115=1.d0/15.d0)
      include 'commons8.inc'	

      dimension psi(*),wgt2(*),wk0(*),wk1(*),wk2(*),wk3(*),wk4(*),  
     &     wk5(*),wk6(*),wk7(*),  
     &     rhot(*),rhotup(*),rhotdown(*),
     &     grhot(*),grhotup(*),grhotdown(*)

      do i=1,mxsize
         rhotup(i)  =0.d0
         rhotdown(i)=0.d0
      enddo
      
c     calculate total densities due to up and down spins
      do iorb=1,norb
         iborb=i1b(iorb)
         isiorb=i1si(iorb)
         nmut=i1mu(iorb)
         
         call exocc (iorb,ocup,ocdown)

         call prod2 (isiorb,psi(iborb),psi(iborb),wk1)
         call dscal (isiorb,ocup,wk1,1)
         
         call prod2 (isiorb,psi(iborb),psi(iborb),wk2)
         call dscal (isiorb,ocdown,wk2,1)
       
c        store total spin densities 
         call add(isiorb,wk1,rhotup)
         call add(isiorb,wk2,rhotdown)
      enddo

      do i=1,mxsize
         wk1(i)=rhotup(i)**const43
         wk2(i)=rhotdown(i)**const43
      enddo	  
         
      call add (mxsize,wk1,wk2)

c     take care of f4 factor
      call multf4(wk2)
         
      wxalpha=fdften(alphaf)*ddot(mxsize,wgt2,1,wk2,1)

c     calculate (nabla rho nabla rho)

c FIXME
      call nfng (rhotup,rhotup,wk0,wk1,wk2,
     &     wk3,wk4,wk5,wk6,wk7)
      call dcopy(mxsize,wk7,ione,grhotup,ione)

      call nfng (rhotdown,rhotdown,wk0,wk1,wk2,
     &     wk3,wk4,wk5,wk6,wk7)
      call dcopy(mxsize,wk7,ione,grhotdown,ione)

      exbe88=wxalpha
     &        +exbe88sup(wgt2,rhotup,grhotup,wk0,wk1) 
     &        +exbe88sup(wgt2,rhotdown,grhotdown,wk0,wk1)
      
      return
      end
