********************************************************************************
*                                                                              * 
*  2DHF version 2-2006                                                         *
*  Copyright (C) 2006 Jacek Kobus                                              *
*                                                                              *
*  This software may be used and distributed according to the terms            *
*  of the GNU General Public License, see README and COPYING.                  *
*                                                                              *
********************************************************************************
c ### fbe88sup ###

c     Calculates the Becke exchange potential for a given density and
c     returns it in grhot array (see Johnson, Gill, Pople, JCP 98 (1993)
c     p.5623) and L.Fan and T.Ziegler, JCP 94 (1991) 6057)

      subroutine fbe88sup (rhot,grhot,wk,wk0,wk1,wk2,wk3,wk4,
     &     wk5,wk6,wk7)

      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      parameter (const13=1.d0/3.d0,const23=2.d0/3.d0,const32=3.d0/2.d0,
     &     const34=3.d0/4.d0,
     &     const43=4.d0/3.d0,const53=5.d0/3.d0,
     &     const115=1.d0/15.d0,bbeta=0.0042)
      parameter(iformA6=2)

      include 'commons8.inc'	

      dimension wk(*),wk0(*),wk1(*),wk2(*),wk3(*),wk4(*),
     &     wk5(*),wk6(*),wk7(*),rhot(*),grhot(*)

c arcsinh
      ash(s)=log(s+sqrt(one+s*s))


c     Calculate |(nabla rhot nabla rhot)| (grhot)
      call nfng (rhot,rhot,wk0,wk1,wk2,wk3,wk4,wk5,wk6,grhot)

c     calculate gamma (see JGP, p.5623)
      do i=1,mxsize
         if (abs(rhot(i)).lt.precis) then
            wk(i)=0.d0
         else
            wk(i)=sqrt(grhot(i))/rhot(i)**const43
         endif
      enddo

c     nabla^2 rho
      call n2f (rhot,wk0,wk1,wk2,grhot)

c     nabla gamma nabla rho (wk7)
      call nfng (wk,rhot,wk0,wk1,wk2,wk3,wk4,wk5,wk6,wk7)

c     calculate g and g prime
      do i=1,mxsize
         g1=fdftpot(alphaf)*rhot(i)**const13

         s=wk(i)
         s2=s*s

         if (abs(rhot(i)).lt.precis) then
            wk1(i)=g1
         else
            fm1=(one+6.d0*bbeta*s*ash(s))
            if (fm1.lt.precis) then
               f=0.d0
            else
               f=one/fm1
            endif

            g3=s/sqrt(one+s2)
        
            t1=const43*s2*rhot(i)**const53
c           nabla^2 rho
            t2=one+f*(one-6.d0*bbeta*s*g3)

c           nabla rho nabla gamma
            t3=6.d0*bbeta*f*((one+two*f)*ash(s)+
     &           g3*(one/(one+s2)+two*f*(two-6.d0*bbeta*s*g3)))
            
            wk1(i)=g1-bbeta*f/rhot(i)**const43*
     &           (t1-grhot(i)*t2+wk7(i)*t3)

         endif
      enddo	  

      call dcopy(mxsize,wk1,ione,grhot,ione)

      return
      end

