********************************************************************************
*                                                                              * 
*  2DHF version 2-2006                                                         *
*  Copyright (C) 1996  Jacek Kobus, Leif Laaksonen, Dage Sundholm              *
*                                                                              *
*  This software may be used and distributed according to the terms            *
*  of the GNU General Public License, see README and COPYING.                  *
*                                                                              *
********************************************************************************
c ### flypcs ###
c
c     Calculates the LYP correlation potential using closed-shell formula 

c	$Id: flyp.f,v 1.2 2006/06/20 17:55:08 jkob Exp jkob $	

      subroutine flypcs (psi,f4,rhot,rhotup,rhotdown,
     &     grhot,grhotup,grhotdown,wk0,wk1,wk2,wk3,wk4,
     &     wk5,wk6,wk7)

      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      parameter (const13=1.d0/3.d0,const43=4.d0/3.d0,
     &     const115=1.d0/15.d0)
      include 'commons8.inc'	

      dimension psi(*),f4(*),wk0(*),wk1(*),wk2(*),wk3(*),wk4(*),
     &     wk5(*),wk6(*),wk7(*),rhot(*),rhotup(*),rhotdown(*),
     &     grhot(*),grhotup(*),grhotdown(*)

      
      do i=1,mxsize
         rhotup(i)   =0.d0
         rhotdown(i) =0.d0
      enddo	  
      
c     calculate total densities due to up and down spins
      do iorb=1,norb
         iborb=i1b(iorb)
         isiorb=i1si(iorb)
         nmut=i1mu(iorb)
         
         call exocc (iorb,ocup,ocdown)

         call prod2 (isiorb,psi(iborb),psi(iborb),wk1)
         call dscal (isiorb,ocup,wk1,1)
         
         call prod2 (isiorb,psi(iborb),psi(iborb),wk2)
         call dscal (isiorb,ocdown,wk2,1)
       
c        store total densities 
         call add(isiorb,wk1,rhotup)
         call add(isiorb,wk2,rhotdown)
      enddo

      call dcopy(mxsize,rhotup,ione,rhot,ione)
      call add(mxsize,rhotdown,rhot)

      call flypsupcs (rhot,rhotup,rhotdown,grhot,grhotup,grhotdown,
     &     wk0,wk1,wk2,wk3,wk4,wk5,wk6,wk7)

      call prod (mxsize,f4,wk7)
      call dcopy(mxsize,wk7,ione,wk2,ione)

      return
      end



********************************************************************************
*                                                                              * 
*  2DHF version 2-2006                                                         *
*  Copyright (C) 1996  Jacek Kobus, Leif Laaksonen, Dage Sundholm              *
*                                                                              *
*  This software may be used and distributed according to the terms            *
*  of the GNU General Public License, see README and COPYING.                  *
*                                                                              *
********************************************************************************
c ### flypsup ###
c
c     Calculates the correlation potential of Lee, Yang and Parr (PRB 37
c     (1988) 785); it is returned in wk10
c

c     eq. 25 gives the contribution due to the density of alpha electrons
c     below the corresponding terms for the beta electrons have been added
c     (note that 1/2 factor ascales these contributions)

      subroutine flypsupcs (rhot,rhotup,rhotdown,wk0,wk1,wk2,wk3,wk4,
     &     wk5,wk6,wk7,wk8,wk9,wk10)

      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      parameter (const13=1.d0/3.d0,const23=2.d0/3.d0,const32=3.d0/2.d0,
     &     const34=3.d0/4.d0,
     &     const43=4.d0/3.d0,const53=5.d0/3.d0,
     &     const115=1.d0/15.d0,const83=8.d0/3.d0,
     &     const19=1.d0/9.d0,const118=1.d0/18.d0,
     &     const49=4.d0/9.d0)

c     coefficients of the Colle-Salveti formula 
      parameter(a=0.04918d0,b=0.132d0,c=0.2533d0,d=0.349d0)


      include 'commons8.inc'	

      dimension wk0(*),wk1(*),wk2(*),wk3(*),wk4(*),
     &     wk5(*),wk6(*),wk7(*),wk8(*),wk(9),wk10(*),
     &     rhot(*),rhotup(*),rhotdown(*)


      cf=0.3d0*(three*pii*pii)**const23
      t23=two**const23
      t53=-a*b*cf
      ab4=-a*b/four
      ab36=-a*b/36.d0
      ab72=-a*b/72.d0
      ab12=-a*b/12.d0
      ab9=-a*b/9.d0
      ab1918=-19.d0*a*b/18.d0
      ab199=-19.d0*a*b/9.d0
      ab724=-7.d0*a*b/24.d0

c      print *,cf,t23,t53,ab4,ab36,ab12,ab9


c FIXME
      do i=1,mxsize
         wk10(i)=0.d0
      enddo

c     total density in rhot
c     F1 (wk0)
c     G1 (wk1)
      do i=1,mxsize
         if (abs(rhot(i)).lt.precis) then
c ???
            wk0(i)=0.d0
         else
            rh=rhot(i)**(-const13)
            wk0(i)=one/(one+d*rh)
            wk1(i)=wk0(i)*rhot(i)**(-const53)*exp(-c*rh)
         endif
      enddo	  
c <OK

c G1 (rhotup)
      call dcopy (mxsize,wk1,ione,rhotup,ione)      

c     (1/2) -abCf (8/3) G1 rhot**(5/3)
      do i=1,mxsize
         wk4(i)=rhot(i)**const53
      enddo

      call prod  (mxsize,wk1,wk4)
      call dscal (mxsize,t53*const83,wk4,ione)
      call add   (mxsize,wk4,wk10)

c     -a F1
      call daxpy (mxsize,-a,wk0,ione,wk10,ione)      

c-------------------------------------------------------------------

c     total density in rhot
c     F1  (wk0)
c     F1' (wk1)
c     G1' (wk2)
      do i=1,mxsize
         if (abs(rhot(i)).lt.precis) then
            wk1(i)=0.d0
            wk2(i)=0.d0
         else
            rh1=rhot(i)**(-const13)
            rh2=(one+d*rh1)
c !!! extra d
            wk1(i)=const13*d*rhot(i)**(-const43)/(rh2*rh2)

            wk2(i)=(wk1(i)-const53*wk0(i)/rhot(i) +
     &              c/three*wk0(i)*rhot(i)**(-const43))*
     &                  rhot(i)**(-const53)*exp(-c*rh1)
         endif
      enddo	  

c     F1' (wk0)
c     G1' (wk1)

      call dcopy (mxsize,wk1,ione,wk0,ione)      
      call dcopy (mxsize,wk2,ione,wk1,ione)      

c G1' (rhotdown)
      call dcopy (mxsize,wk1,ione,rhotdown,ione)      

c < OK

c     -a F1' rhot
      call prod  (mxsize,rhot,wk0)
      call daxpy (mxsize,-a,wk0,ione,wk10,ione)      
      

c     (1/2) -abCf (8/3) G1' rhot**(5/3)
      do i=1,mxsize
         wk4(i)=rhot(i)**const53
      enddo

      call prod  (mxsize,rhot,wk1)
      call prod  (mxsize,wk1,wk4)
      call dscal (mxsize,t53*const83,wk4,ione)
      call add   (mxsize,wk4,wk10)

c <OK

c     potential is collected in wk10

C c     terms with -19/18 ab G1 nabla^2 rhot 
       call n2f (rhot,wk0,wk1,wk2,wk3)

       call dcopy (mxsize,wk3,ione,wk4,ione)      

       call prod  (mxsize,rhotup,wk3)
       call dscal (mxsize,ab1918,wk3,ione)
       call add (mxsize,wk3,wk10)

       call dcopy (mxsize,wk4,ione,wk3,ione)
      
c -19/9 ab rhot nabla^2 rhot
       call prod  (mxsize,rhot,wk3)
       call dscal (mxsize,ab199,wk3,ione)
       call add (mxsize,wk3,wk10)

C c     -19/9 ab G1' nabla rho nabla rho
       call nfng (rhot,rhot,wk2,wk3,wk4,wk5,wk6,wk7,wk8,wk9)
       call prod  (mxsize,rhotdown,wk9)
       call dscal (mxsize,ab199,wk9,ione)
       call add (mxsize,wk9,wk10)

       return
c     total density in rhot
c     F1   (wk0)
c     g0   (wk1)
c     g1   (wk2)
c     G1'' (wk3)
      do i=1,mxsize
         if (abs(rhot(i)).lt.precis) then
c ???
            wk0(i)=one
         else
            rh=rhot(i)**(-const13)
            wk0(i)=one/(one+d*rh)
            wk1(i)=rhot(i)**(-const53)*exp(-c*rh)
            wk2(i)=-const53+c/three*rhot(i)**(-const13)
         endif
      enddo	

      do i=1,mxsize
         if (abs(rhot(i)).lt.precis) then
c ???
            wk3(i)=0.d0
         else
            wk3(i)=two*d*d/9.0d0*rhot(i)**(-const23)*wk0(i)*wk0(i)
     &           +const49*d*(wk2(i)-one)*rhot(i)**(-const13)*wk0(i)
     &           +wk2(i)*(wk2(i)-one)-d/three*rhot(i)**(-const13)
            wk3(i)=wk0(i)*wk1(i)*rhot(i)**(-two)*wk3(i)
         endif
      enddo	
  
      call nfng (rhot,rhot,rhotup,rhotdown,wk4,wk5,wk6,wk7,wk8,wk9)
      call prod  (mxsize,rhot,wk9)
      call prod  (mxsize,wk3,wk9)
      call dscal (mxsize,ab724,wk9,ione)
      call add (mxsize,wk9,wk10)

      return
      end

