********************************************************************************
*                                                                              * 
*  2DHF version 2-2006                                                         *
*  Copyright (C) 1996  Jacek Kobus, Leif Laaksonen, Dage Sundholm              *
*                                                                              *
*  This software may be used and distributed according to the terms            *
*  of the GNU General Public License, see README and COPYING.                  *
*                                                                              *
********************************************************************************
c ### flypsup ###

c     Calculates the correlation potential of Lee, Yang and Parr (PRB 37
c     (1988) 785); it is returned in wk10
c

c     Eq. 25 gives the contribution due to the density of alpha electrons.
c     Check the formula!
c     Below the corresponding terms for the beta electrons have been added.
c     (note that 1/2 factor scales these contributions)

c     $Id: flypsup.f,v 1.3 2006/06/28 09:40:30 jkob Exp $	

      subroutine flypsup (rhot,rhotup,rhotdown,wk0,wk1,wk2,wk3,wk4,
     &     wk5,wk6,wk7,wk8,wk9,wk10)

      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      parameter (const13=1.d0/3.d0,const23=2.d0/3.d0,const32=3.d0/2.d0,
     &     const34=3.d0/4.d0,
     &     const43=4.d0/3.d0,const53=5.d0/3.d0,
     &     const115=1.d0/15.d0,const83=8.d0/3.d0,
     &     const19=1.d0/9.d0,const118=1.d0/18.d0)

c     coefficients of the Colle-Salveti formula 
      parameter(a=0.04918d0,b=0.132d0,c=0.2533d0,d=0.349d0)


      include 'commons8.inc'	

      dimension wk0(*),wk1(*),wk2(*),wk3(*),wk4(*),
     &     wk5(*),wk6(*),wk7(*),wk8(*),wk(9),wk10(*),
     &     rhot(*),rhotup(*),rhotdown(*)


      cf=0.3d0*(three*pii*pii)**const23
      t23=two**const23
      t53=-two**const53*a*b*cf
      ab4=-a*b/four
      ab36=-a*b/36.d0
      ab12=-a*b/12.d0
      ab9=-a*b/9.d0

c      print *,cf,t23,t53,ab4,ab36,ab12,ab9


c FIXME
      do i=1,mxsize
         wk10(i)=0.d0
      enddo

c      return
c     total density in rhot
c     F2 (wk0)
c     G2 (wk1)
      do i=1,mxsize
         if (abs(rhot(i)).lt.precis) then
            wk0(i)=two
         else
            t=two*
     &           (one-
     &                (rhotup(i)*rhotup(i)+rhotdown(i)*rhotdown(i))/
     &                 (rhot(i)*rhot(i))
     &           )
            rh=rhot(i)**(-const13)
            wk0(i)=t/(one+d*rh)
            wk1(i)=wk0(i)*rhot(i)**(-const53)*exp(-c*rh)
         endif
      enddo	  
c <OK

c     potential is collected in wk10

c     terms with nabla^2 G2
      call n2f (wk1,wk0,wk1,wk2,wk3)
c     -ab/4 rho nabla^2 G2  (wk3)
      call dcopy (mxsize,wk3,ione,wk4,ione)
      call prod  (mxsize,rhot,wk4)
      call dscal (mxsize,ab4,wk4,ione)
      call add (mxsize,wk4,wk10)

c     (1/2) -ab/12 rhoup nabla^2 G2
      call dcopy (mxsize,wk3,ione,wk4,ione)      
      call prod  (mxsize,rhotup,wk4)
      call dscal (mxsize,half*ab12,wk4,ione)
      call add (mxsize,wk4,wk10)


c     (1/2) -ab/12 rhodown nabla^2 G2
      call dcopy (mxsize,wk3,ione,wk4,ione)      
      call prod  (mxsize,rhotdown,wk4)
      call dscal (mxsize,half*ab12,wk4,ione)
      call add (mxsize,wk4,wk10)


c     -ab nabla G2 nabla rho
      call nfng (wk1,rhot,wk2,wk3,wk4,wk5,wk6,wk7,wk8,wk9)
      call dscal (mxsize,ab,wk9,ione)

      call add (mxsize,wk9,wk10)

c     (1/2) -ab nabla G2 nabla rhoup
      call nfng (wk1,rhotup,wk2,wk3,wk4,wk5,wk6,wk7,wk8,wk9)
      call dscal (mxsize,half*ab9,wk9,ione)
      call add (mxsize,wk9,wk10)


c     (1/2) -ab nabla G2 nabla rhodown
      call nfng (wk1,rhotdown,wk2,wk3,wk4,wk5,wk6,wk7,wk8,wk9)
      call dscal (mxsize,half*ab9,wk9,ione)
      call add (mxsize,wk9,wk10)

c     -ab G2 nabla^2 rhot 
      call n2f   (rhot,wk2,wk3,wk4,wk5)
      call dcopy (mxsize,wk5,ione,wk4,ione)      
      call prod  (mxsize,wk1,wk4)
      call dscal (mxsize,ab,wk4,ione)
      call add   (mxsize,wk4,wk10)


c     (1/2) -ab/9 G2 nabla^2 rhotup 
      call n2f   (rhotup,wk2,wk3,wk4,wk5)
      call dcopy (mxsize,wk5,ione,wk4,ione)      
      call prod  (mxsize,wk1,wk4)
      call dscal (mxsize,half*ab9,wk4,ione)
      call add   (mxsize,wk4,wk10)


c     (1/2) -ab/9 G2 nabla^2 rhotdown 
      call n2f   (rhotdown,wk2,wk3,wk4,wk5)
      call dcopy (mxsize,wk5,ione,wk4,ione)      
      call prod  (mxsize,wk1,wk4)
      call dscal (mxsize,half*ab9,wk4,ione)
      call add   (mxsize,wk4,wk10)


c     (1/2) -2**(5/3)abCf (8/3) G2 rhotup**(5/3)
      do i=1,mxsize
         wk4(i)=rhotup(i)**const53
         wk5(i)=rhotdown(i)**const53
      enddo

      call prod  (mxsize,wk1,wk4)
      call dscal (mxsize,half*t53*const83,wk4,ione)
      call add   (mxsize,wk4,wk10)

c     (1/2) -2**(5/3)abCf (8/3) G2 rhotdown**(5/3)
      call prod  (mxsize,wk1,wk5)
      call dscal (mxsize,half*t53*const83,wk5,ione)
      call add   (mxsize,wk5,wk10)

c     -a F2
      call daxpy (mxsize,-a,wk0,ione,wk10,ione)      

c-------------------------------------------------------------------

c     total density in rhot
c     F2  (wk0)
c     F2' (wk1)
c     G2' (wk2)
      do i=1,mxsize
         if (abs(rhot(i)).lt.precis) then
            wk1(i)=0.d0
            wk2(i)=0.d0
         else
            t=two*
     &           (one-
     &                (rhotup(i)*rhotup(i)+rhotdown(i)*rhotdown(i))/
     &                 (rhot(i)*rhot(i))
     &           )
            rh1=rhot(i)**(-const13)
            rh2=(one+d*rh1)
c forgoten d
            wk1(i)=const13*t*d*rhot(i)**(-const43)/(rh2*rh2)

            wk2(i)=(wk1(i)-const53*wk0(i)/rhot(i) +
     &              c/three*wk0(i)*rhot(i)**(-const43))*
     &                  rhot(i)**(-const53)*exp(-c*rh1)
         endif
      enddo	  

c     F2' (wk0)
c     G2' (wk1)

      call dcopy (mxsize,wk1,ione,wk0,ione)      
      call dcopy (mxsize,wk2,ione,wk1,ione)      

c----------------------------------------------------------------------

c     nabla rhot nabla rhot
      call nfng  (rhot,rhot,wk2,wk3,wk4,wk5,wk6,wk7,wk8,wk9)
      call dscal (mxsize,-one,wk9,ione)

c     rhot nabla^2 rhot 
      call n2f  (rhot,wk2,wk3,wk4,wk5)
      call prod (mxsize,rhot,wk5)

c     (rhot nabla^2 rhot) - (nabla rhot nabla rhot)
      call add (mxsize,wk5,wk9)

c     times ab/4 G2'
      call prod  (mxsize,wk1,wk9)
      call dscal (mxsize,ab4,wk9,ione)
      call add   (mxsize,wk9,wk10)


c     -ab/36 G2' (nabla rhotup nabla rhotup)
      call nfng  (rhotup,rhotup,wk2,wk3,wk4,wk5,wk6,wk7,wk8,wk9)
      call prod  (mxsize,wk1,wk9)
      call dscal (mxsize,ab36,wk9,ione)
      call add   (mxsize,wk9,wk10)


c     -ab/36 G2' (nabla rhotdown nabla rhotdown)
      call nfng  (rhotdown,rhotdown,wk2,wk3,wk4,wk5,wk6,wk7,wk8,wk9)
      call prod  (mxsize,wk1,wk9)
      call dscal (mxsize,ab36,wk9,ione)
      call add   (mxsize,wk9,wk10)



c     -ab/36 G2' (nabla rhotdown nabla rhotdown)
      call nfng  (rhotdown,rhotdown,wk2,wk3,wk4,wk5,wk6,wk7,wk8,wk9)
      call prod  (mxsize,wk1,wk9)
      call dscal (mxsize,ab36,wk9,ione)
      call add   (mxsize,wk9,wk10)


c     rhotup nabla^2 rhotup 
      call n2f  (rhotup,wk2,wk3,wk4,wk5)
      call prod (mxsize,rhotup,wk5)

c     rhotdown nabla^2 rhotdown 
      call n2f  (rhotdown,wk2,wk3,wk4,wk6)
      call prod (mxsize,rhotdown,wk6)


c     -ab/36 3 G2' (rhotup nabla^2 rhotup + rhotdown nabla^2 rhotdown)
      call add   (mxsize,wk5,wk6)
      call prod  (mxsize,wk1,wk6)
      call dscal (mxsize,ab36*three,wk6,ione)
      call add   (mxsize,wk6,wk10)


c     -2**(5/3)abCf     G2' (rhotup**(8/3)+rhotdown**(8/3))???
c     -2**(5/3)abCf  1/2   G2' (rhotup**(8/3)+rhotdown**(8/3))
      do i=1,mxsize
         wk4(i)=rhotup(i)**const83+rhotdown(i)**const53
      enddo
      call prod  (mxsize,wk1,wk4)
      call dscal (mxsize,t53,wk4,ione)
      call dscal (mxsize,half,wk4,ione)
      call add   (mxsize,wk4,wk10)

c     -aF2' rhot
      call dcopy (mxsize,rhot,ione,wk4,ione)
      call prod  (mxsize,wk0,wk4)
      call dscal (mxsize,-a,wk4,ione)
      call add   (mxsize,wk4,wk10)

      return
      end

