********************************************************************************
*                                                                              *
*  2DHF version 2-2006                                                         *
*  Copyright (C) 1996  Jacek Kobus, Leif Laaksonen, Dage Sundholm              *
*                                                                              *
*  This software may be used and distributed according to the terms            *
*  of the GNU General Public License, see README and COPYING.                  *
*                                                                              *
********************************************************************************
c ### fock ###
c
c     Calculates the Fock potential for a given orbital.

c     $Id: fock.f,v 1.3 2006/06/28 08:40:43 jkob Exp $	

      subroutine fock (iorb,psi,pot,excp,e,f0,f1,f2,f4,wk0,wk1,wk2)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      include 'commons8.inc'
      dimension psi(*),pot(*),excp(*),e(*),f0(*),f1(*),f2(*),f4(*),
     &	    wk0(*),wk1(*),wk2(*)	

      iborb=i1b(iorb)
      ngorb=i1si(iorb)
      ibpot=i2b(iorb)
      ngpot=i2si(iorb)	
      norb2=norb*norb		
      ipe=mgx(6,iorb)
      ipex=ipe-2*(ipe/2)
      
      if (ipe.eq.0) then
         call dcopy (mxsize,f0,1,wk0,1)
         call daxpy (mxsize,eng(iorb),f1,1,wk0,1)
      else

c        e enter the expression with minus sign which is already
c        incorporated in e

         w=dble(ipe*ipe)
         call dcopy (mxsize,f0,1,wk0,1)
         call daxpy (mxsize,eng(iorb),f1,1,wk0,1)
         call daxpy (mxsize,w,e,1,wk0,1)
      endif

      do i=1,mxsize
         wk1 (i)=0.d0
         wk2 (i)=0.d0	  
         foc2(i)=0.d0
      enddo	

      if (nel.gt.1) then
         if (norb.eq.1) then

c FIXME what about one non-sigma orbital            
c           contribution from coulomb potential of one sigma orbital

            call dcopy (ngpot,pot(ibpot),1,wk1,1)
         else

c           add contributions from Coulomb and exchange potentials

            do iorb1=1,norb
c            do iorb1=norb,1,-1
               iborb1=i1b(iorb1)
               ngorb1=i1si(iorb1)
               kex=iorb+norb*(iorb1-1)
               ideng=iorb1+norb*(iorb-1)
               
               if (iorb.le.iorb1) then
                  idexp=iorb+iorb1*(iorb1-1)/2
               else
                  idexp=iorb1+iorb*(iorb-1)/2
               endif
               
               ibpot1=i2b(iorb1)
               ngpot1=i2si(iorb1)
               ibexp=i3b(idexp)
               ngexp=i3si(idexp)

               coo=occ(iorb1)
               
C                if (idbg(95).ne.0) then
C                   write(*,*) 'fock: iorb,iorb1,coo',iorb,iorb1,coo
C                endif

c FIXME               
               if (iorb.eq.iorb1) coo=coo-1.d0
               coo0=coo

               call daxpy (ngpot1,coo,pot(ibpot1),1,wk1,1)
               
               coo1=0.d0
               coo2=0.d0

               if (iorb1.ne.iorb)  then
                  coo=gec(kex)
                  coo1=coo
                  call dcopy (ngexp,excp(ibexp),1,wk2,1)
                  call dscal (ngexp,coo,wk2,1)
                  if (ilc(idexp).gt.1) then
                     coo=gec(kex+norb2)
                     coo2=coo
                     call daxpy (ngexp,coo,excp(ibexp+ngexp),1,wk2,1)
                  endif
                  
                  if (engo(ideng).ne.0.d0) then
                     call daxpy (ngexp,engo(ideng),f4,1,wk2,1)
                  endif
                  
                  call prod (ngorb1,psi(ibpot1),wk2)
                  call add  (ngorb1,wk2,foc2)
               else
                  if ((ipe.gt.0).and.(ilc(idexp).gt.0)) then
                     coo=gec(kex)
                     coo2=coo
                     call dcopy (ngexp,excp(ibexp),1,wk2,1)
                     ngrid=max(ngexp,ngorb1)
c                     print *,'fock: iorb,iorb1,ibexp ',
c     &                    iorb,iorb1,ibexp
c FIXME
                     if (idbg(92).eq.0) then
                        call prod  (ngorb1,psi(iborb1),wk2)
                        call dscal (ngrid,coo,wk2,1)
                        call add (ngrid,wk2,foc2)
                     else
                        call dscal (ngrid,-coo,wk2,1)
                        call add (ngrid,wk2,wk1)
                     endif
                  endif
               endif

               if (idbg(95).ne.0) then
                  write(*,1000) iorn(iorb),bond(iorb),gut(iorb),               
     &                 iorn(iorb1),bond(iorb1),gut(iorb1),
     &                 coo0,coo1,coo2,engo(ideng)
 1000             format(4x,i2,1x,a5,1x,a1,1x,i2,1x,a5,1x,a1,1x,
     &                 4f8.3)     
               endif
            enddo
         endif
         
         if (idbg(97).ne.0) then
            write(*,*) 'fock: coulomb potential'
            call pmtx (nni,nmu(1),wk1,1,1,incrni,incrmu)
            write(*,*) 'fock: one-electron contr.'
            call pmtx (nni,nmu(1),wk0,1,1,incrni,incrmu)
         endif
         
c        multiply Coulomb and exchange potentials by f2
         
         call prod (mxsize,f2,wk1)
         call prod (mxsize,f2,foc2)
      endif

c  add one electron and coulomb pot. contributions
c  are coulomb and exchange pot zero at infinity?

      call add   (mxsize,wk0,wk1)
      call dcopy (mxsize,wk1,1,foc1,1)

      return
      end


