********************************************************************************
*                                                                              *
*  2DHF version 2-2006                                                         *
*  Copyright (C) 1996  Jacek Kobus, Leif Laaksonen, Dage Sundholm              *
*                                                                              *
*  This software may be used and distributed according to the terms            *
*  of the GNU General Public License, see README and COPYING.                  *
*                                                                              *
********************************************************************************
c ### focksl ###

c     Calculates the exchange potentials as the local Slater approximation.

      subroutine focksl(iorb,psi,pot,excp,e,f0,f1,f2,f4,
     &     wk0,wk1,wk2,wk3,wk4,wk5,wk6,wk7,wk8,wk9,wk10,wk11,wk12,wk13)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      parameter (const13=1.d0/3.d0)
      include 'commons8.inc'	
      dimension psi(*),pot(*),excp(*),e(*),f0(*),f1(*),f2(*),f4(*),
     &	    wk0(*),wk1(*),wk2(*),wk3(*),wk4(*),wk5(*),
     &      wk6(*),wk7(*),wk8(*),wk9(*),wk10(*),wk11(*),wk12(*),wk13(*)
	
c     contributions from one-electron terms
      iborb=i1b(iorb)
      ngorb=i1si(iorb)
      ibpot=i2b(iorb)
      ngpot=i2si(iorb)	
      norb2=norb*norb			

      ipe=mgx(6,iorb)
      ipex=ipe-2*(ipe/2)
      
      if (ipe.eq.0) then
	 call dcopy (mxsize,f0,ione,wk0,ione)
	 call daxpy (mxsize,eng(iorb),f1,ione,wk0,ione)
      else

c        e enter the expression with minus sign which is already
c        incorporated in e

         w=dble(ipe*ipe)
         call dcopy (mxsize,f0,ione,wk0,ione)
         call daxpy (mxsize,eng(iorb),f1,ione,wk0,ione)
         call daxpy (mxsize,w,e,ione,wk0,ione)
      endif

c     store wk0 in foc2 for a time being
      call dcopy (mxsize,wk0,ione,foc2,ione) 

      do i=1,mxsize
         wk1 (i)=0.d0
         wk2 (i)=0.d0
      enddo	

c     contributions from the local slater exchange

      if (nel.gt.1) then

c LDA
         if (idftex.eq.1) then

            do iorb1=1,norb
               iborb1 =i1b (iorb1)
               isiorb1=i1si(iorb1)	  
               call exocc (iorb1,ocup,ocdown)
               call prodas (isiorb1,ocup , psi(iborb1),psi(iborb1),wk1)
               call prodas (isiorb1,ocdown,psi(iborb1),psi(iborb1),wk2)
            enddo
            
            do i=1,mxsize
               wk1(i)=(wk1(i))**const13
               wk2(i)=(wk2(i))**const13
            enddo	  
            
            call add (mxsize,wk1,wk2)
            
c           FIXME 
c           multiply the local exchange potential by f4 to make it
c           commensurate with the coulomb potential

            tmpf=fdftpot(alphaf)
            do i=1,mxsize
               wk2(i)=tmpf*wk2(i)*f4(i)
            enddo		

c B88
         elseif (idftex.eq.2) then
            call fbe88(psi,f4,wk8,wk9,wk10,wk11,wk12,wk13,wk0,wk1,wk2,
     &           wk3,wk4,wk5,wk6,wk7)

c PW86
         elseif (idftex.eq.3) then
            call fpw86(psi,f4,wk8,wk9,wk10,wk11,wk12,wk13,wk0,wk1,wk2,
     &           wk3,wk4,wk5,wk6,wk7)

c PW91
         elseif (idftex.eq.4) then
            write(*,'(/"Error: PW91 exchange potential is not ",
     &                 "supported")')
            stop 'focksl'

c            call fpw91(psi,f4,wk8,wk9,wk10,wk11,wk12,wk13,wk0,wk1,wk2,
c     &           wk3,wk4,wk5,wk6,wk7)
         elseif (idftex.gt.4) then
            write(*,'(/"Warning: Unsupported exchange potential")')
            stop 'focksl'
         endif

c        store the local exchange potential in a separate array
         call dcopy (mxsize,wk2,ione,flda,ione)

c FIXME
         do i=1,mxsize
            wk2 (i)=0.d0
         enddo	
 
c        add contributions from correlations potentials
c LYP
         if (idftcorr.eq.1) then
            call flypcs(psi,f4,wk8,wk9,wk10,wk11,wk12,wk13,wk0,wk1,wk2,
     &           wk3,wk4,wk5,wk6,wk7)

c VWN
         elseif (idftcorr.eq.2) then
            call fvwncs(psi,f4,wk8,wk9,wk10,wk11,wk12,wk13,wk0,wk1,wk2,
     &           wk3,wk4,wk5,wk6,wk7)

         elseif (idftcorr.gt.2) then
            write(*,'(/"Warning: Unsupported correlation potential")')
            stop 'focksl'

         endif
         
      endif

c     add correlation contributions to the exchange ones
      call add (mxsize,flda,wk2)
      
      call dcopy (mxsize,foc2,ione,wk0,ione) 

      do i=1,mxsize
         wk1 (i)=0.d0
         foc2(i)=0.d0
      enddo	

      if (nel.gt.1) then

c        add contributions from Coulomb and off-diagonal Lagrange multipliers.

         do iorb1=1,norb
            iborb1=i1b(iorb1)	   	 
            ngorb1=i1si(iorb1)	 
            ibpot1=i2b(iorb1)
            ngpot1=i2si(iorb1)
            
            ideng=iorb1+norb*(iorb-1)
            coo=occ(iorb1)
            
c           in the local exchange approximation the Coulomb potential also
c           includes the contribution from the orbital
            
            if (idbg(95).ne.0) then
               write(*,'("focksl: iorb1,ibpot1,coo",2i7,d12.4)')
     &              iorb1,ibpot1,coo
            endif
            call daxpy (ngpot1,coo,pot(ibpot1),ione,wk1,ione)
            
            if (iorb.ne.iorb1.and.engo(ideng).ne.0.d0) then
               do i=1,ngorb1
                  foc2(i)=foc2(i)+engo(ideng)*f4(i)*psi(iborb1+i-1)
               enddo		
            endif
         enddo
      
c        store the local exchange potential in a separate array (to be used by raylsl)
         call dcopy (mxsize,wk2,ione,flda,ione)
         
c        add the local exchange to the coulomb potential
         call add (mxsize,wk2,wk1)
         
c        multiply coulomb/exchange potentials and off-diagonal Lagrange
c        multipliers by f2

         call prod (mxsize,f2,wk1)
         call prod (mxsize,f2,foc2)
c        nel.gt.1 
      endif
      
c     add one-electron to Coulomb and exchange contributions

      call add   (mxsize,wk0,wk1)
      call dcopy (mxsize,wk1,ione,foc1,ione)

      return
      end


