
#include <math.h>
double AM_HK_ZSimpson(double x, double y, double z, double e0, double w, 
		      double V0, double a, double Nt);


double poth3_(double *z, double *s, int *m, double *V0, double *a)

// Funkcja zwracajca warto potencjau typu -V0/Sqrt(a^2+s^2+z^2)
// w punkcie (z,s) - ukad wsprzdnych cylindrycznych -
// plus poprawka wynikajca z zamiany kartezjaskiego 
// ukadu wsprzdnych na ukad cylindryczny
//
// PARAMETRY
// z,s - wsprzdne cylindryczne
// m - magnetyczna liczba kwantowa
// V0 - gboko potencjau (V0>0)
// a - szeroko rdzenia
//	jeli a=0, V0=1 -> potencja kulombowski
//	jeli a>0, V0-dowolne -> potencja modelowy
{
	double popr_cylind = (*s==0.0?(*m * *m)/(2*0.0000000001):(*m * *m)/(2 * *s * *s));

	return -*V0/sqrt(*a * *a+*z * *z+*s * *s)+popr_cylind;
}



double potkh_(double *z, double *s, int *m, double *eps, double *w, 
	      double *V0, double *a, double *N)

// Funkcja zwracajca warto potencjau Kramersa - Hennebergera
// w punkcie (z,s) - ukad wsprzdnych cylindrycznych - 
// dla pierwotnego potencjau atomu modelowego
// plus poprawka wynikajca z zamiany kartezjaskiego 
// ukadu wsprzdnych na ukad cylindryczny 
//
// PARAMETRY
// z,s - wsprzdne cylindryczne
//   m - magnetyczna liczba kwantowa
// eps - amplituda pola elektrycznego
//   w - czsto pola elektrycznego
//  V0 - gboko pierwotnego potencjau (V0>0)
//   a - szeroko rdzenia pierwotnego potencjau
//	 UWAGA!!! Parametr a musi by wikszy od zera
//   N - liczba przedziaw w numerycznym cakowaniu  
{
    	double popr_cylind = (*s==0.0?(*m * *m)/(2*0.0000000001):(*m * *m)/(2 * *s * *s));

	//        return AM_HK_ZSimpson(*z, *s, 0.0, *eps, *w, *V0, *a, *N)+popr_cylind;
        return AM_HK_ZSimpson(*z, *s, 0.0, *eps, *w, *V0, *a, *N);
	
}

// FUNKACJE POMOCNICZE
// -------------------------------------------------
// potencja HK w postaci cakowej
double AM_HK(double t, double x, double y, double z, double a, double V0, double e0, double w)
{
    double alpha0=e0/(w*w), bx=x+alpha0+alpha0*(cos(w*t)-1);
    return -V0/((2*M_PI/w)*sqrt(a*a+bx*bx+y*y+z*z));
}


// numeryczne cakowanie zoon metod Simpsona
double AM_HK_ZSimpson(double x, double y, double z, double e0, double w, 
		      double V0, double a, double Nt)
{
    double T=2*M_PI/w, h=T/Nt;
    double parts[3];
    double t=T-h;
    int i,k;
    parts[0]=parts[1]=0;
    parts[2]=AM_HK(0,x,y,z,a,V0,e0,w)+AM_HK(T,x,y,z,a,V0,e0,w);
    for (i=Nt,k=0; --i; t-=h,k=!k)
    	parts[k] += AM_HK(t,x,y,z,a,V0,e0,w);

    return (parts[2]+2*parts[1]+4*parts[0])*h/3;
}
