********************************************************************************
*                                                                              *
*  2DHF version 2-2006                                                         *
*  Copyright (C) 1996  Jacek Kobus, Leif Laaksonen, Dage Sundholm              *
*                                                                              *
*  This software may be used and distributed according to the terms            *
*  of the GNU General Public License, see README and COPYING.                  *
*                                                                              *
********************************************************************************
c ### lagra ###
c
c	$Id: lagra.f,v 1.6 2006/02/03 18:59:32 jkob Exp $	
c
c     Calculates the off-diagonal lagrange multipliers.
c     Damping factors are set to 1, i.e. dmp=1
c 
      subroutine lagra (iorb,psi,pot,excp,e,f0,wgt1,wgt2,
     &			wk0,wk1,wk2,wk3)

      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      parameter (ffthresh=5.d-02)
      include 'commons8.inc'
      dimension psi(*),pot(*),excp(*),e(*),f0(*),wgt1(*),wgt2(*),
     &     wk0(*),wk1(*),wk2(*),wk3(*)

      if (nel.eq.1)  return
      if (norb.le.1) return
      if (iorb.eq.norb) return

c FIXME
      iorbbeg=iorb+1
      do iorb1=1,norb
         if (ifix(iorb1).ne.0) iorbbeg=1
      enddo

      do iorb1=iorbbeg,norb

c        if break is on and the two orbitals have different symmetry
c        off-diagonal lm is not calculated

c        if (ihomon.eq.2.and.ihomo(iorb1)*ihomo(iorb).lt.0) goto 10
         if (ihomo(iorb1)*ihomo(iorb).lt.0) goto 10

         if (nlm(iorb1,iorb).eq.0) goto 10

         ipc1=iorb1+(iorb-1)*norb
         ipc2=iorb+(iorb1-1)*norb

c        store previous values in order to make damping of LM possible
         engoprv1=engo(ipc1)
         engoprv2=engo(ipc2)

c        In HF calculations occupation numbers are integers and thus
c        divt assumes either zero or nonzero value. The threshold used
c        is appropriate for a multiconfiguration version of the code (as
c        suggested by Froese-Fisher in her atomic MCHF code)

c        There are two possibilities to calculate Lagrange multipliers.
c        If abs((occ(iorb)-occ(iorb1))/(occ(iorb)*occ(iorb1)) is
c              > 5.e-02  take <j/h/i> - <i/h/j>  
c              < 5.e-02  take <j/h/i> + <i/h/j>

         if (abs(divt).lt.ffthresh) then
            if (lmtype.eq.0) then 
               divt=(occ(iorb1)+occ(iorb))/(occ(iorb1)*occ(iorb))
               call rayln1 (iorb1,iorb,psi,pot,excp,e,f0,wgt1,wgt2,
     &              wk0,wk1,wk2,wk3)
               call rayln1 (iorb,iorb1,psi,pot,excp,e,f0,wgt1,wgt2,
     &              wk0,wk1,wk2,wk3)
               ent=(engo(ipc1)+engo(ipc2))/divt
               engo1=ent/occ(iorb)
               engo2=ent/occ(iorb1)
            elseif (lmtype.eq.1) then
               call rayln1 (iorb1,iorb,psi,pot,excp,e,f0,wgt1,wgt2,
     &              wk0,wk1,wk2,wk3)
               call rayln1 (iorb,iorb1,psi,pot,excp,e,f0,wgt1,wgt2,
     &              wk0,wk1,wk2,wk3)
               engo1=engo(ipc1)
               engo2=engo(ipc2)
            elseif (lmtype.eq.2) then
               call rayln2 (iorb1,iorb,psi,pot,excp,wgt2,wk0,wk1)
               call rayln2 (iorb,iorb1,psi,pot,excp,wgt2,wk0,wk1)
               engo1=engo(ipc1)
               engo2=engo(ipc2)
            else
               write(iout6,1000)
 1000          format(/1x,'... off-diagonal Lagrange multiplies',
     &              ' cannot be calculated ...'//)
               stop 'lagra'
            endif
         else
            divt=(occ(iorb1)-occ(iorb))/(occ(iorb1)*occ(iorb))
            call rayln2 (iorb1,iorb,psi,pot,excp,wgt2,wk0,wk1)
            call rayln2 (iorb,iorb1,psi,pot,excp,wgt2,wk0,wk1)
            ent=(engo(ipc1)-engo(ipc2))/divt
            engo1=ent/occ(iorb)
            engo2=ent/occ(iorb1)
         endif	   	   

         engo(ipc1)=sflagra*((1.d0-dflagra)*engo1+dflagra*engoprv1)
         engo(ipc2)=sflagra*((1.d0-dflagra)*engo2+dflagra*engoprv2)

c FIXME
         if (idbg(333).ne.0) then
            write(*,*) 'lagra: ',lmtype,sflagra,
     &           iorn(iorb),bond(iorb1),iorn(iorb1),bond(iorb1),
     &           engo1,engo2
         endif

 10      continue
      enddo

      return
      end


 
