********************************************************************************
*                                                                              * 
*  2DHF version 2-2006                                                         *
*  Copyright (C) 2006 Jacek Kobus                                              *
*                                                                              *
*  This software may be used and distributed according to the terms            *
*  of the GNU General Public License, see README and COPYING.                  *
*                                                                              *
********************************************************************************
c ### nfng ###

c     Calculates nabla f nabla g; the result is returned in wk3

c     $Id: nfng.f,v 1.5 2006/06/28 10:19:33 jkob Exp $	

      subroutine nfng(f,g,fmu,fni,gmu,gni,wk0,wk1,wk2,wk3)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      include 'commons8.inc'	

      dimension f(*),g(*),fmu(*),fni(*),gmu(*),gni(*),
     &     wk0(*),wk1(*),wk2(*),wk3(*)

c     it is assumed that functions have isym=1 symmetry
      isym=1

c     calculate df/dmu and df/dni derivatives

      call fill   (nni,mxnmu,isym,f,wk3)
      call dif1ni (mxnmu,wk3,wk0,wk1,wk2)
      call refill (nni,mxnmu,fni,wk0)
      call dif1mu (mxnmu,wk3,wk2)
      call refill (nni,mxnmu,fmu,wk2)

c     calculate dg/dmu and dg/dni derivatives
      call fill   (nni,mxnmu,isym,g,wk3)
      call dif1ni (mxnmu,wk3,wk0,wk1,wk2)
      call refill (nni,mxnmu,gni,wk0)
      call dif1mu (mxnmu,wk3,wk2)
      call refill (nni,mxnmu,gmu,wk2)
         
c     calculate df/dmu dg/dmu term (wk0)
      call prod2(mxsize,fmu,gmu,wk0)
      
      do i=1,mxnmu
         ii=(i-1)*nni
         if (vxi1(i).lt.precis) then
            do j=1,nni
               wk0(ii+j)=0.d0
            enddo
         else
            do j=1,nni
               z=(half*r)*vxi(i)*veta(j)
               wk0(ii+j)=wk0(ii+j)*
     &              ((half*r*vxi1(i)*veta1(j)*vxi(i))**2 + 
     &               (vxi(i)*z-half*r*veta(j))**2)/vxi1(i)**2
            enddo
         endif
      enddo

c     calculate df/dni dg/dni term (wk1)
      call prod2(mxsize,fni,gni,wk1)

      do i=1,mxnmu
         ii=(i-1)*nni
         do j=1,nni
            if (veta1(j).lt.precis) then
               wk1(ii+j)=0.d0
            else
               z=(half*r)*vxi(i)*veta(j)
               wk1(ii+j)=wk1(ii+j)*
     &              ((half*r*vxi1(i)*veta1(j)*veta(j))**2 +
     &              (half*r*vxi(i)-veta(j)*z)**2)/veta1(j)**2
            endif
         enddo
      enddo


c     calculate df/dmu dg/dni+ df/dni dg/dmu term (wk2)

      call prod2(mxsize,fmu,gni,wk2)
      call prod2(mxsize,fni,gmu,wk3)
      call add  (mxsize,wk2,wk3)

      do i=1,mxnmu
         ii=(i-1)*nni
         do j=1,nni
              if (veta1(j).lt.precis.or.vxi1(i).lt.precis) then
               wk3(ii+j)=0.d0
            else
               z=(half*r)*vxi(i)*veta(j)
               wk3(ii+j)=wk3(ii+j)*
     &            ( (half*r*vxi1(i)*veta1(j))**2*vxi(i)*veta(j)-
     &            (half*r*vxi(i)-veta(j)*z)*(vxi(i)*z-half*r*veta(j)))/
     &            (vxi1(i)*veta1(j))
            endif
         enddo
      enddo

      call add  (mxsize,wk0,wk3)
      call add  (mxsize,wk1,wk3)
      
      do i=1,mxnmu
         ii=(i-1)*nni
         do j=1,nni
            t1=r*r*(vxi(i)*vxi(i)-veta(j)*veta(j))
            if (t1.lt.precis) then
               wk3(ii+j)=0.d0
            else
               wk3(ii+j)=wk3(ii+j)*(four/t1)**2
            endif
c            write (*,'(2i4,e15.4)') i,j, wk3(ii+j)
         enddo
      enddo

      return
      end

c ### testnfng ###
c     Tests nabla f nabla g

      subroutine testnfng(rhot,grhot,wk0,wk1,wk2,wk3,wk4,
     &     wk5,wk6,wk7)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      include 'commons8.inc'	

      dimension wk0(*),wk1(*),wk2(*),wk3(*),wk4(*),
     &     wk5(*),wk6(*),wk7(*),rhot(*),grhot(*)

      itest=1

      if (itest.eq.1) then
         do i=1,mxnmu
            ii=(i-1)*nni
            do j=1,nni
               rhot(ii+j) =0.d0
               grhot(ii+j)=0.d0
            enddo
         enddo
      
         call nfng (rhot,grhot,wk0,wk1,wk2,wk3,wk4,wk5,wk6,wk7)
         call pmtx(nni,mxnmu,wk7,1,1,incrni,incrmu)

      endif
      
      return
      end
