********************************************************************************
*                                                                              * 
*  2DHF version 2-2006                                                         *
*  Copyright (C) 2005  Jacek Kobus                                             *
*                                                                              *
*  This software may be used and distributed according to the terms            *
*  of the GNU General Public License, see README and COPYING.                  *
*                                                                              *
********************************************************************************
c ### nuclder ####

c     This subroutine calculates the derivative of the orbital density
c     over z (along the internuclear distance) at (0,0,-R/2).  At this
c     point the derivative cannot be calculated in (\nu,\mu) variables
c     since the denominators of the derivative expresion are zero though
c     the derivative over z is finite at the point.  The derivative has
c     to be calculated for non -1 values of \nu and then extrapolated to
c     \nu=-1.

c     $Id: nuclder.f,v 1.2 2006/06/28 20:44:38 jkob Exp $	
   
      subroutine nuclder(psi,e,f0,wgt1,wgt2,
     &			dermu,dernu,wk2,wk3,wk4,wk5)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      include 'commons8.inc'

      dimension psi(nni,*),e(*),f0(*),wgt1(*),wgt2(*),
     &          dermu(nni,*),dernu(nni,*),wk2(*),wk3(*),wk4(*),wk5(*)
      dimension dznu(5000),dzmu(6),tderzorb1(60),tderzorb2(60),
     &      tderzorb3(60),tderzorb4(60)
      iprt=0

c     prepare differentialtion arrays used by difni and difmu
      
      if (nni.gt.5000) then
         write(*,*) 'nuclder: nni > 5000 '
         stop 'nuclder'
      endif

      call prepdiff1
      
c     calculate the total electron density
     
c     orbital density functions are of even symmetry

      write(*,*)
      write(*,*) 'derivative and derivative/(N*N) of density ',
     &     ',over z at (0,0,-R/2) for orbital '
      write(*,*)
      isym=1
      tderz=0.d0
      nmut=mxnmu

c     der of density as 2*|psi>*d|psi>/dz

      do iorb=1,norb

c        for hydrogen-like systems the accuracy of the derivatives can
c        be checked since the analytical formulae are known For example,
c        the 1s orbital density is N^2*exp(-2Zr) and the derivative is
c        -2Z up to the normalization constant. This procedure calculates
c        the derivative and prints outs both its value and the value
c        divided by N squared.
c
         n1=mgx(1,iorb)
         l1=mgx(2,iorb)
         m1=mgx(3,iorb)
         ez1=eza1(iorb)
     
c normalization factor for Laguere polynomials
         
         fn1=(2.d0*ez1/dble(n1))**(3.d0/2.d0+dble(l1))
     &        *sqrt(factor(n1+l1)/(2.d0*dble(n1)*factor(n1-l1-1)))
     &        /factor(2*l1+1)

c normalization factor for spherical harmonics

         shn1=(-1.d0)**dble(m1)/sqrt(4.d0*pii)*
     &        sqrt((2*l1+1)*factor(l1-m1)/factor(l1+m1))
         if (m1.eq.0) 	  shn1=1.d0/sqrt(4.d0*pii)*
     &        sqrt((2*l1+1)*factor(l1-m1)/factor(l1+m1))

         fn=fn1*shn1

         i1beg=i1b(iorb)
         nmut=i1mu(iorb)
         ib=(iorb-1)*nmut+1
         call nuclder1(iorb,psi(1,ib),dznu,dzmu,dernu,dermu,
     &        wk2,wk3,wk4,wk5)

        tderz=tderz+dznu(nni)+dzmu(1)
        tderzorb1(iorb)=dznu(nni)
        tderzorb2(iorb)=dzmu(1)
        tderzorb3(iorb)=dznu(nni)+dzmu(1)
        tderzorb4(iorb)=dznu(nni)/(fn*fn)
      enddo

      write(*,*)
      write(*,*) 'derivative of the total density',tderz
      write(*,*) 'and its components '
      do iorb=1,norb
         write(*,1000) iorn(iorb),bond(iorb),tderzorb1(iorb),
     &        tderzorb2(iorb),tderzorb3(iorb),tderzorb4(iorb)
      enddo

      write(*,*)
      write(*,*)

c FIXME
c===========================
      tderz=0.d0
      do iorb=1,norb

c  for hydrogen-like systems the accuracy of the derivatives can be
c  checked since the analytical formulae are known
c  For example, the 1s orbital density is N^2*exp(-2Zr) and the 
c  derivative is -2Z up to the normalization constant. This procedure 
c  calculates the derivative and prints outs both its value and the 
c  value divided by N squared.
c
         n1=mgx(1,iorb)
         l1=mgx(2,iorb)
         m1=mgx(3,iorb)
         ez1=eza1(iorb)
c     
c normalization factor for Laguere polynomials
c
         fn1=(2.d0*ez1/dble(n1))**(3.d0/2.d0+dble(l1))
     &        *sqrt(factor(n1+l1)/(2.d0*dble(n1)*factor(n1-l1-1)))
     &        /factor(2*l1+1)
c
c normalization factor for spherical harmonics
c
         shn1=(-1.d0)**dble(m1)/sqrt(4.d0*pii)*
     &        sqrt((2*l1+1)*factor(l1-m1)/factor(l1+m1))
         if (m1.eq.0) 	  shn1=1.d0/sqrt(4.d0*pii)*
     &        sqrt((2*l1+1)*factor(l1-m1)/factor(l1+m1))
c
         fn=fn1*shn1
c
         i1beg=i1b(iorb)
         nmut=i1mu(iorb)
         ngorb=i1si(iorb)
         ngpot=i2si(iorb)	
c     
         call tden(iorb,ngorb,psi,wk2)
c
         call dcopy(ngorb,wk2,1,psi(1,1),1)
c 
         call fill (nni,nmut,isym,psi(1,1),wk3)	
         call difni (nmut,wk3,dermu,dernu,wk2)
         call refill(nni,nmut,dernu,dermu)
c
	call difmu (nmut,wk3,wk2)
	call refill(nni,nmut,dermu,wk2)
c

c  derivatives at the right nuclei (0,0,R/2)
c
c       dernu(1,1)= exeven(1)*dernu(2,1)+exeven(2)*dernu(3,1)
c     &          +exeven(3)*dernu(4,1)
c     &          +exeven(4)*dernu(5,1)+exeven(5)*dernu(6,1)
c
c        dermu(1,1)= exeven(1)*dermu(1,2)+exeven(2)*dermu(1,3)
c     &          +exeven(3)*dermu(1,4)
c     &          +exeven(4)*dermu(1,5)+exeven(5)*dermu(1,6)
c
c        write(*,*) '(dernu(i,1),i=1,6) after extrapolation'
c        write(*,*) (dernu(i,1),i=1,6)
c
c        write(*,*) 'difmu'
c        write(*,*) (dermu(1,i),i=1,6)
c
        dernu(nni,1)= exeven(1)*dernu(nni-1,1)
     &       +exeven(2)*dernu(nni-2,1)
     &       +exeven(3)*dernu(nni-3,1)
     &       +exeven(4)*dernu(nni-4,1)
     &       +exeven(5)*dernu(nni-5,1)
c
        dermu(nni,1)= exeven(1)*dermu(nni,2)
     &       +exeven(2)*dermu(nni,3)
     &       +exeven(3)*dermu(nni,4)
     &       +exeven(4)*dermu(nni,5)
     &       +exeven(5)*dermu(nni,6)
c

c  vxi1(i)=sqrt(xi^2-1)
c  veta1(i)=sqrt(1-eta^2)
c
c  having derivative over nu calculate the derivative over z
c  near (0,0,-R/2)
c
	do inu=nni-5,nni
	  dznu(inu)=(-1.d0)*2.d0/(r*veta1(inu))*dernu(inu,1)
 	enddo
c
	dzmu(1)=0.d0
	do imu=2,6
    	  dzmu(imu)=(-1.d0)*2.d0/(r*vxi1(imu))*dermu(nni,imu)
 	enddo	
c
c   calculate the derivative over z at (0,0,R/2+) by extrapolation
c
        dznu(nni)= exeven(1)*dznu(nni-1)+exeven(2)*dznu(nni-2)
     &       +exeven(3)*dznu(nni-3)
     &       +exeven(4)*dznu(nni-4)+exeven(5)*dznu(nni-5)
c
c   calculate the derivative over z at (0,0,ER/2-) by extrapolation
c
        dzmu(1)= exeven(1)*dzmu(2)+exeven(2)*dzmu(3)
     &       +exeven(3)*dzmu(4)
     &       +exeven(4)*dzmu(5)+exeven(5)*dzmu(6)
c
        iprt=0
        if (iprt.eq.0) then        
           write(*,*) 'dznu(nni) ',(dznu(i),i=nni-5,nni)
           write(*,*) 'dzmu(1)   ',(dzmu(i),i=1,6)
        endif
c
        write(*,*) iorn(iorb),'  ',bond(iorb),  
     &       dznu(nni),dznu(nni)/(fn*fn),dznu(nni)*(fn*fn)
c
        tderz=tderz+dznu(nni)+dzmu(1)
        tderzorb1(iorb)=dznu(nni)
        tderzorb2(iorb)=dzmu(1)
        tderzorb3(iorb)=dznu(nni)+dzmu(1)
        tderzorb4(iorb)=dznu(nni)/(fn*fn)
      enddo
c
      write(*,*)
      write(*,*) 'derivative of the total density',tderz
      write(*,*) 'and its components '
      do iorb=1,norb
         write(*,1000) iorn(iorb),bond(iorb),tderzorb1(iorb),
     &        tderzorb2(iorb),tderzorb3(iorb),tderzorb4(iorb)
      enddo


01000 format(1x,i2,a6,2x,4e20.10)
      write(*,*)
      return
      end
c ===============================================================
      subroutine nuclder1(iorb,psi,dznu,dzmu,dernu,dermu,
     &     wk2,wk3,wk4,wk5)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      include 'commons8.inc'
      dimension psi(nni,*),dznu(5000),dzmu(6)
      dimension dermu(nni,*),dernu(nni,*),wk2(*),wk3(*),wk4(*),wk5(*)
c
      i1beg=i1b(iorb)
      nmut=i1mu(iorb)
      ngorb=i1si(iorb)
      ngpot=i2si(iorb)	
c
c   interpolate psi at (nni,1)
c
      psi(nni,1)= exeven(1)*psi(nni-1,1)+exeven(2)*psi(nni-2,1)
     &     +exeven(3)*psi(nni-3,1)
     &     +exeven(4)*psi(nni-4,1)+exeven(5)*psi(nni-5,1)
c     
      call fill (nni,nmut,isym,psi,wk2)	
      call difni (nmut,wk2,wk3,wk4,wk5)
      call refill(nni,nmut,dernu,wk3)
c
      call difmu (nmut,wk2,wk3)
      call refill(nni,nmut,dermu,wk3)
c
      dernu(nni,1)= exeven(1)*dernu(nni-1,1)
     &     +exeven(2)*dernu(nni-2,1)
     &     +exeven(3)*dernu(nni-3,1)
     &     +exeven(4)*dernu(nni-4,1)
     &     +exeven(5)*dernu(nni-5,1)
c
      dermu(nni,1)= exeven(1)*dermu(nni,2)
     &     +exeven(2)*dermu(nni,3)
     &     +exeven(3)*dermu(nni,4)
     &     +exeven(4)*dermu(nni,5)
     &     +exeven(5)*dermu(nni,6)
c

c  vxi1(i)=sqrt(xi^2-1)
c  veta1(i)=sqrt(1-eta^2)
c
c  having derivative over nu calculate the derivative over z
c  near (0,0,-R/2)
c
      do inu=nni-5,nni
         dznu(inu)=(-1.d0)*2.d0/(r*veta1(inu))*dernu(inu,1)
      enddo
c
      dzmu(1)=0.d0
      do imu=2,6
         dzmu(imu)=(-1.d0)*2.d0/(r*vxi1(imu))*dermu(nni,imu)
      enddo	
c
c   calculate the derivative over z at (0,0,R/2+) by extrapolation
c
      dznu(nni)= exeven(1)*dznu(nni-1)+exeven(2)*dznu(nni-2)
     &     +exeven(3)*dznu(nni-3)
     &     +exeven(4)*dznu(nni-4)+exeven(5)*dznu(nni-5)
c     
c   calculate the derivative over z at (0,0,ER/2-) by extrapolation
c
      print *,'nucder1-9'
      dzmu(1)= exeven(1)*dzmu(2)+exeven(2)*dzmu(3)
     &     +exeven(3)*dzmu(4)
     &     +exeven(4)*dzmu(5)+exeven(5)*dzmu(6)
c
c FIXME
      iprt=1
      if (iprt.eq.0) then        
         write(*,*) 'psi(nni-5,...nni,1)'
         write(*,2000) (psi(i,1),i=nni-5,nni)
         write(*,*) 'dznu(nni-5,nni)'
         write(*,2000) (dznu(i),i=nni-5,nni)
         write(*,*) 'psi(5,..1)'
         write(*,2000) (psi(nni,i),i=6,1,-1)
         write(*,*) 'dzmu(6,...,1)'
         write(*,2000) (dzmu(i),i=6,1,-1)
02000    format(1x,3e25.12/1x,3e25.12)
      endif
c
      dznu(nni)=occ(iorb)*2.d0*psi(nni,1)*dznu(nni)
      dzmu(1)  =occ(iorb)*2.d0*psi(nni,1)*dzmu(1)
c
      return
      end

c ### tden ###
c    this routine initializes arrays used for differentiation
c    needed in nuclder
c
      subroutine tden(iorb,ngorb1,psi,wk2)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      include 'commons8.inc'
      dimension psi(*),wk2(*)
c
      do i=1,ngorb1
         wk2(i)=0.d0
      enddo
c
      call extinorg(psi)

c      do iorb1=1,norb
      do iorb1=iorb,iorb
         iborb1=i1b(iorb1)	   	 
         ngorb1=i1si(iorb1)	 	 
         coo=occ(iorb1)
         do i=1,ngorb1
            wk2(i)=wk2(i)+coo*psi(iborb1+i-1)*psi(iborb1+i-1)
         enddo
      enddo
c 
      return
      end
c### extinorg ###
      subroutine extinorg(psi)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      include 'commons8.inc'
c
      dimension psi(nni,*)
c
      iprt=0
      if (iprt.eq.0) then        
         write(*,*) 'psi(nni-5..,1) ',(psi(i,1),i=nni-5,nni)
         write(*,*) 'psi(nni,1..)   ',(psi(nni,i),i=1,6)
      endif

      psi(1,1)= exeven(1)*psi(2,1)+exeven(2)*psi(3,1)
     &          +exeven(3)*psi(4,1)
     &          +exeven(4)*psi(5,1)+exeven(5)*psi(6,1)

        psi(nni,1)= exeven(1)*psi(nni-1,1)+exeven(2)*psi(nni-2,1)
     &          +exeven(3)*psi(nni-3,1)
     &          +exeven(4)*psi(nni-4,1)+exeven(5)*psi(nni-5,1)
c
      if (iprt.eq.0) then        
         write(*,*)
         write(*,*) 'psi(nni-5..,1) ',(psi(i,1),i=nni-5,nni)
         write(*,*) 'psi(nni,1..)   ',(psi(nni,i),i=1,6)
      endif
      return
      end


c ### prepdiff1 ###
c    this routine initializes arrays used for differentiation
c    needed in nuclder
c
      subroutine prepdiff1 
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      include 'commons8.inc'
      dimension aa1(9),aa2(9),a1(9),a2(9)
c 
c  derivative coeff. from 8th-order Stirling interpolation formula
c
      data aa1/ 3.d0, -32.d0, 168.d0, -672.d0, 0.d0, 672.d0,
     &	     -168.d0,  32.d0,	-3.d0 /
      data aa2/  -9.d0,  128.d0, -1008.d0, 8064.d0, -14350.d0,
     &	       8064.d0,-1008.d0,   128.d0,   -9.d0 /
c
c  initialization of the array performing differentiation
c  based on the 9-point interpolating formula
c
      w1=1.d0/(840.d0)
      w2=1.d0/(5040.d0)
c
c   initialize dni1, dni2, dmu1 and dmu2
c
      do ig=1,ngrids
      do  i=1,4
        dmu2(i,ig)=w2*aa2(i)/(hmu(ig)*hmu(ig))
        dmu1(i,ig)=w1*aa1(i)/hmu(ig)
      enddo
      enddo
c
      do i=1,4
        dni2(i)=w2*aa2(i)/(hni*hni)
        dni1(i)=w1*aa1(i)/hni
      enddo
c
c  initialize arrays used in matrix differentiation
c
c  initialize dmu
c
      ib=1
      do ig=1,ngrids
c
        do k=1,9
          a2(k)=w2*aa2(k)/(hmu(ig)*hmu(ig))
          a1(k)=w1*aa1(k)/hmu(ig)
        enddo 
c
        ie=iemu(ig)
        do i=ib,ie
        do k=1,9
c          dmu(k,i)=a2(k)+borb(1,i)*a1(k)
          dmu(k,i)=a1(k)
        enddo
        enddo
        ib=ie+1
      enddo
c
c  initialize dni 
c
      do k=1,9
        a2(k)=w2*aa2(k)/(hni*hni)
        a1(k)=w1*aa1(k)/hni
      enddo
c
      do i=1,nni
      do k=1,9
c        dni(k,i)=a2(k)+d(i,1)*a1(k)
        dni(k,i)=a1(k)
      enddo
      enddo
c
      return
      end





