********************************************************************************
*                                                                              *
*  2DHF version 2-2006                                                         *
*  Copyright (C) 1996  Jacek Kobus, Leif Laaksonen, Dage Sundholm              *
*                                                                              *
*  This software may be used and distributed according to the terms            *
*  of the GNU General Public License, see README and COPYING.                  *
*                                                                              *
********************************************************************************
c ### oshell ###

c     Calculates the weights of two-electron (exchange) contributions to
c     the total energy expression for an open shell scf case.

c     $Id: oshell.f,v 1.2 2006/06/28 10:21:48 jkob Exp $	


      subroutine oshell
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      character*8 alpha,beta
      include 'commons8.inc'
      data alpha/'+'/, beta/'-'/

      do i=1,2*maxorb*maxorb
         gec(i)=0.d0
      enddo

      do i=1,norb
         do j=1,norb
            nlm(i,j)=0
         enddo
      enddo

      do i=1,4*norb
         spn(i)=spin(i)
      enddo

      do i=1,norb
         isumi=4*(i-1)
*        gec(i)=occ(i)-1.0
         iloop(i)=2
         iqm(1+isumi)=1
         iqm(2+isumi)=1
         if (mgx(3,i).gt.0) then
            iloop(i)=4
            iqm(3+isumi)=-1
            iqm(4+isumi)=-1
         endif
      enddo
      
c     initialize spn and spin arrays
c     throw out the electrons + = alpha and - = beta
c     the orbital is locked = 1 (open shell case)
c                           = 0 (close shell case) 
      
      do i=1,norb
         if (lock(i).eq.0) then
            jj=4*(i-1)
            spn(jj+1)=alpha
            spn(jj+2)=beta
            if (mgx(3,i).gt.0) then
               spn(jj+3)=alpha
               spn(jj+4)=beta
            endif
         endif
      enddo
      
      do i=1,4*norb
         spin(i)=spn(i)
      enddo
      
c     loop now through all electrons
      
      isu2=norb*norb
      do i=1,norb
         ixx=4*(i-1)
         div(i)=0.d0
         do ii=1,iloop(i)
            ix=ii+ixx
            if ((spn(ix).ne.alpha).and.(spn(ix).ne.beta)) goto 12
            div(i)=div(i)+1.d0
            
            do j=1,norb
               jxx=4*(j-1)
               kk=i+norb*(j-1)

               mpl=abs(mgx(3,i)-mgx(3,j))
               mmi=abs(mgx(3,i)+mgx(3,j))

               do jj=1,iloop(j)
                  jx=jxx+jj
                  if ((spn(jx).ne.alpha).and.(spn(jx).ne.beta)) goto 15
c                 div(i)=div(i)+1.0d0
                  
                  if ((mpl.eq.mmi).and.(spn(ix).eq.spn(jx)).
     &                 and.(i.ne.j)) then
                     gec(kk)=gec(kk)+1.d0
                  endif
                  
                  if ((mpl.ne.mmi).and.(spn(ix).eq.spn(jx))) then
                     if ((i.eq.j).and.(iqm(ix).ne.iqm(jx))) then
                        gec(kk)=gec(kk)+1.d0
                     endif

                     if ((i.ne.j).and.(iqm(ix).eq.iqm(jx))) then
                        gec(kk)=gec(kk)+1.d0
                     endif
                     
                     if ((i.ne.j).and.(iqm(ix).ne.iqm(jx))) then
                        gec(kk+isu2)=gec(kk+isu2)+1.d0
                     endif
                  endif
 15               continue
               enddo
            enddo
 12         continue
         enddo
         
         if (abs(div(i)).lt.1.d-06) goto 1000
         
         if (idbg(150).ne.0) then
            write(6,1101)
01101       format(2x,' i,idiv,gec(kk),gec(kk+isu2),div(i)')
         endif
         do idiv=1,norb
            kk=i+norb*(idiv-1)
            gec(kk)=gec(kk)/div(i)
            gec(kk+isu2)=gec(kk+isu2)/div(i)
            if (idbg(150).ne.0) then
c               gec(kk)=1.0*gec(kk)
               write(6,1102)  iorn(i),bond(i),gut(i),
     &              iorn(idiv),bond(idiv),gut(idiv),
     &              gec(kk),gec(kk+isu2),div(i)
01102          format(i4,1x,a8,a1,i4,1x,a8,a1,3f7.3)
            endif
         enddo
 10      continue
      enddo
            
      ial=0
      ibe=0
      imag=0
      do i=1,norb
         isumi=4*(i-1)
         if (lock(i).eq.0) goto 100

         do ii=1,iloop(i)
            imu=ii+isumi
            if (spn(imu).eq.alpha) ial=ial+1
            if (spn(imu).eq.beta) ibe=ibe+1
            if (spn(imu).eq.alpha.or.spn(imu).eq.beta) then
               imag=imag+iqm(imu)
            endif
 110        continue
         enddo
 100     continue
      enddo

c     Non-zero entries of nlm array indicate pairs of orbitals having
c     non-zero off-diagonal Lagrange multipliers. Each pair of orbitals
c     of the same symmetry is examined to see if an orthogonal
c     transformation can be applied and off-diagonal Lagrange
c     multipliers set to zero. One can make the multipliers to appear in
c     Fock equations by specifying the pair of orbitals in question via
c     largraon label.

      do j=1,(norb-1)
         jxx=4*(j-1)
         do i=(j+1),norb
            nlm(j,i)=0
            nlm(i,j)=0

            if (mgx(6,j).ne.mgx(6,i)) goto 400
            if (gut(j).ne.gut(i)) goto 400

            if (iloop(j).ne.iloop(i)) goto 400
            ixx=4*(i-1)
            match=0
            do ii=1,iloop(i)
               ix=ii+ixx
               jx=ii+jxx
               if (spn(ix).eq.spn(jx)) match=match+1
            enddo
            if (match.ne.iloop(i)) then
               nlm(j,i)=1
               nlm(i,j)=1
            endif
 400        continue
         enddo
      enddo

      do j=1,(norb-1)
         do i=(j+1),norb
            do k=1,2*norb,2
               if (lagraon(k).eq.j.and.lagraon(k+1).eq.i) then
                  nlm(j,i)=1
                  nlm(i,j)=1
               endif
            enddo
         enddo
      enddo
      
c     Testing mode only
c     Select a way in which off-diagonal Lagrange multipliers are
c     calculated between closed shell orbitals (i.e. when lagraon label
c     is present).

      lmtype=0
      if (idbg(336).eq.1) lmtype=1
      if (idbg(337).eq.1) lmtype=2

      return

 1000 continue
      write(iout6,1001)
 1001 format(/1x,'... error in spin orbitals ...'//)
      stop 'oshell'
      end
