********************************************************************************
*                                                                              *
*  2DHF version 2-2006                                                         *
*     Copyright (C) 2006 Jacek Kobus                                           *
*                                                                              *
*  This software may be used and distributed according to the terms            *
*  of the GNU General Public License, see README and COPYING.                  *
*                                                                              *
********************************************************************************
c ### radialden ###

c     Calculates total radial densities relative to centres A (z-R/2)
c     and B (z+R/2) along the internuclear axis (from A to -infity or B
c     to +infity).

c     $Id: radialden.f,v 1.2 2006/06/28 20:31:03 jkob Exp $	

      subroutine radialden(psi)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      parameter (const13=1.d0/3.d0)
      include 'commons8.inc'	
      dimension psi(*)

      ngorb=i1si(norb)
      do i=1,ngorb
         flda(i)=0.d0
      enddo
	 	       
      do iorb=1,norb
         iborb=i1b(iorb)	   	 
         coo=occ(iorb)
         do i=1,ngorb
            flda(i)=flda(i)+coo*psi(iborb+i-1)*psi(iborb+i-1)
         enddo
      enddo

      iunit=99
      if     (iprint(80).eq.1) then

c        print radial density relative to centre A 
         open(iunit,file='density-A',status='unknown',form='formatted')
         call prtdenA(nni,i1mu(1),flda,iunit)
         close(iunit)

      elseif (iprint(82).eq.1) then

c        print radial density relative to centre B
         open(iunit,file='density-B',status='unknown',form='formatted')
         call prtdenB(ione,i1mu(1),flda,iunit)
         close(iunit)
      endif

      return
      end

c ### prtdenA ###

      subroutine prtdenA (m,n,a,ioutmat)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      include 'commons8.inc'
      dimension a(m,n)

      write(ioutmat,'(10x,"r(au)",12x,"total electronic density")')
      in=m
      do imu=1,n
         r1t=(half*r)*(vxi(imu)+veta(in))
         write(ioutmat,'(2E25.16)') r1t,a(in,imu)
      enddo

      return
      end

c ### prtdenB ###

      subroutine prtdenB (m,n,a,ioutmat)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      include 'commons8.inc'
      dimension a(m,n)

      write(ioutmat,'(10x,"r(au)",12x,"total electronic density")')
      in=1
      do imu=1,n
         r2t=(r/2.d0)*(vxi(imu)-veta(in))
         write(ioutmat,'(2E25.16)') r2t,a(in,imu)
      enddo

      return
      end
