*******************************************************************************
*                                                                              *
*  2DHF version 2-2006                                                         *
*  Copyright (C) 1996  Jacek Kobus, Leif Laaksonen, Dage Sundholm              *
*                                                                              *
*  This software may be used and distributed according to the terms            *
*  of the GNU General Public License, see README and COPYING.                  *
*                                                                              *
********************************************************************************
c ### rayln ###

c     Evaluates the off-diagonal Lagrange multipliers 
c      e(iorb2,iorb1) = e(2,1) = <2|T_k +V_n+V_C-V_x|1>
c      e(iorb1,iorb2) = e(1,2) = <1|T_k +V_n+V_C-V_x|2>

c     $Id: rayln.f,v 1.4 2006/06/28 20:46:42 jkob Exp $	 

c FIXME rayln vs rayln1/2
      subroutine rayln1(iorb1,iorb2,psi,pot,excp,e,f0,wgt1,wgt2,
     &			wk0,wk1,wk2,wk3)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)

      include 'commons8.inc'
      dimension psi(*),pot(*),excp(*),e(*),f0(*),wgt1(*),wgt2(*),
     &     wk0(*),wk1(*),wk2(*),wk3(*)

      if (islat.ne.0) then

c        use the local exchange approximation
         call raynsl(iorb1,iorb2,psi,pot,excp,wgt2,wk0,wk1)
         return
      endif
      
      if (mgx(6,iorb1).ne.mgx(6,iorb2)) return
      if (ige(iorb1).ne.ige(iorb2)) return

      isd=iorb1+(iorb2-1)*norb

      engo(isd)=0.d0

      i1beg1=i1b(iorb1)
      ngorb1=i1si(iorb1)
      ngpot1=i2si(iorb1)	

      i1beg2=i1b(iorb2)
      ngorb2=i1si(iorb2)     
      ngpot2=i2si(iorb2)	

      norb2=norb*norb

      nmut=i1mu(iorb1)
      ipe=mgx(6,iorb1)
      ipex=ipe-2*(ipe/2)

c     calculate derivatives over mu and ni 
      if (ipex.eq.0) then
         isym= 1
      else
         isym=-1
      endif

      call fill (nni,nmut,isym,psi(i1beg1),wk3)
      call difni (nmut,wk3,wk0,wk1,wk2)
      call refill(nni,nmut,wk1,wk0)

      call difmu (nmut,wk3,wk2)
      call refill(nni,nmut,wk0,wk2)

c     add contributions from derivatives over mu and ni
      ngorb=min(ngorb1,ngorb2)
      call add (ngorb,wk0,wk1)

      if (ipe.eq.0) then
         call dcopy (ngorb,f0,ione,wk0,ione)
      else
         
c        nuclear energy for non-sigma orbitals contains contribution
c        from e term (in toten this term is correctly added to the
c        kinetic energy contribution); e enters the expression with
c        minus sign which is already incorporated in e

         w=dble(ipe*ipe)

         call dcopy (ngorb,f0,ione,wk0,ione)
         call daxpy (ngorb,w,e,ione,wk0,ione)
      endif

      call proda (ngorb,psi(i1beg1),wk0,wk1)
      call prod (ngorb,psi(i1beg2),wk1)
      
      woneel=ddot(ngorb,wgt1,ione,wk1,ione)

c     add contributions from Coulomb and exchange potentials

      do iorb=1,norb
         i1beg=i1b(iorb)
         ngorb=i1si(iorb)		
         ngpot=i2si(iorb)
         
         kex=iorb2+norb*(iorb-1)
         if (iorb2.le.iorb) then
            ihc=iorb2+iorb*(iorb-1)/2
         else
            ihc=iorb+iorb2*(iorb2-1)/2
         endif

         i2beg=i2b(iorb)
         i3beg=i3b(ihc)
         ngex =i3si(ihc)
         
         ngrid=min(ngorb2,ngpot)		
         do i=1,mxsize
            wk0(i)=0.d0
         enddo	    

         coo=occ(iorb)
         if (iorb2.eq.iorb) coo=coo-1.d0
         
         call dcopy (ngrid,pot(i2beg),ione,wk0,ione)
         call prod  (ngrid,psi(i1beg2),wk0)

         if (coo.ne.1.d0) then
            call dscal (ngrid,coo,wk0,ione)
         endif

         ngrid2=min(ngorb2,ngex,ngorb)		
         if (iorb.ne.iorb2)  then
            coo=gec(kex)
            call prodas (ngrid2,-coo,psi(i1beg),excp(i3beg),wk0)
            if (ilc(ihc).gt.1) then
               coo=gec(kex+norb*norb)
               call prodas (ngrid2,-coo,psi(i1beg),
     &              excp(i3beg+i3si(ihc)),wk0)
            endif
         else
            if ((ipe.gt.0).and.(ilc(ihc).gt.0)) then
               coo=gec(kex)
               call prodas (ngrid2,-coo,psi(i1beg),excp(i3beg),wk0)
            endif
         endif
       
         if (iorb.eq.1) then
            call dcopy (ngrid,wk0,ione,wk1,ione)
         else
            call add   (ngrid,wk0,wk1)
         endif
      enddo
      
c     to complete the integrand wk1 has to be multiplied by psi(i1beg1)
      call prod (ngrid,psi(i1beg1),wk1)
      wtwoel=ddot(ngrid,wgt2,ione,wk1,ione)

      if (idbg(335).ne.0) then
         print *,'rayln1: woneel,wtoel',woneel,wtwoel
      endif

      engo(isd)= woneel+wtwoel
      
      return
      end

********************************************************************************
*                                                                              *
*  2DHF version 2-2006                                                         *
*  Copyright (C) 1996  Jacek Kobus, Leif Laaksonen, Dage Sundholm              *
*                                                                              *
*  This software may be used and distributed according to the terms            *
*  of the GNU General Public License, see README and COPYING.                  *
*                                                                              *
********************************************************************************
c ### rayln2 ###
c 
c     Evaluates the off-diagonal Lagrange multipliers e(iorb2,iorb1) and
c     e(iorb1,iorb2) in cases occ(iorb1)<>occ(iorb2)
c
      subroutine rayln2(iorb1,iorb2,psi,pot,excp,wgt2,wk0,wk1)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)

      include 'commons8.inc'
      dimension psi(*),pot(*),excp(*),wgt2(*),wk0(*),wk1(*)

c     Slater local exchange approximation is used
      if (islat.ne.0) then
         call raynsl(iorb1,iorb2,psi,pot,excp,wgt2,wk0,wk1)
         return
      endif

      if (mgx(6,iorb1).ne.mgx(6,iorb2)) return
      if (ige(iorb1).ne.ige(iorb2)) return

      isd=iorb1+(iorb2-1)*norb
      ipe=mgx(6,iorb2)
      engo(isd)=0.d0
      i1beg1=i1b(iorb1)
      i1beg2=i1b(iorb2)
      ngorb1=i1si(iorb1)
      ngpot1=i2si(iorb1)	

      ngorb2=i1si(iorb2)     
      ngpot2=i2si(iorb2)	

      norb2=norb*norb

c     add contributions from coulomb and exchange potentials

      do iorb=1,norb
         i1beg=i1b(iorb)
         ngorb=i1si(iorb)		
         ngpot=i2si(iorb)
         
         kex=iorb2+norb*(iorb-1)
         if (iorb2.le.iorb) then
            ihc=iorb2+iorb*(iorb-1)/2
         else
            ihc=iorb+iorb2*(iorb2-1)/2
         endif

         i2beg=i2b(iorb)
         i3beg=i3b(ihc)
         ngex =i3si(ihc)
         
         ngrid=min(ngorb2,ngpot)		
         do i=1,mxsize
            wk0(i)=0.d0
         enddo	    

         coo=occ(iorb)
         if (iorb2.eq.iorb) coo=coo-1.d0
         
         call dcopy (ngrid,pot(i2beg),ione,wk0,ione)
         call prod  (ngrid,psi(i1beg2),wk0)

         if (coo.ne.1.d0) then
            call dscal (ngrid,coo,wk0,ione)
         endif

         ngrid2=min(ngorb2,ngex,ngorb)		
         if (iorb.ne.iorb2)  then
            coo=gec(kex)
            call prodas (ngrid2,-coo,psi(i1beg),excp(i3beg),wk0)
            if (ilc(ihc).gt.1) then
               coo=gec(kex+norb*norb)
               call prodas (ngrid2,-coo,psi(i1beg),
     &              excp(i3beg+i3si(ihc)),wk0)
            endif
         else
            if ((ipe.gt.0).and.(ilc(ihc).gt.0)) then
               coo=gec(kex)
               call prodas (ngrid2,-coo,psi(i1beg),excp(i3beg),wk0)
            endif
         endif
       
         if (iorb.eq.1) then
            call dcopy (ngrid,wk0,ione,wk1,ione)
         else
            call add   (ngrid,wk0,wk1)
         endif
      enddo
      
c     to complete the integrand wk1 has to be multiplied by psi(i1beg1)
      call prod (ngrid,psi(i1beg1),wk1)
      wtwoel=ddot(ngrid,wgt2,ione,wk1,ione)

      if (idbg(335).ne.0) then
         print *,'rayln2: wtwoel',wtwoel
      endif

      engo(isd)= wtwoel
      
      return
      end



