********************************************************************************
*                                                                              * 
*  2DHF version 2-2006                                                         *
*  Copyright (C) 1996  Jacek Kobus, Leif Laaksonen, Dage Sundholm              *
*                                                                              *
*  This software may be used and distributed according to the terms            *
*  of the GNU General Public License, see README and COPYING.                  *
*                                                                              *
********************************************************************************
c ### relorb1 ###  

c     Evaluates the Fock potential for a given orbital, sets up the
c     right-hand side of the Poisson equation for that orbital and
c     performs a few (MC)SOR iterations.

c     $Id: relorb1.f,v 1.3 2006/06/28 08:42:49 jkob Exp $	

      subroutine relorb1 (iorb,cw_sor,psi,pot,excp,b,d,e,
     &     f0,f1,f2,f4,wgt2,
     &     lhs,rhs,wk0,wk1,wk2,wk3,wk4,wk5,wk6,wk7,wk8,wk9,wk10,wk11)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      integer*4 cw_sor
      real*8 lhs
      include 'commons8.inc'
      include 'commons8a.inc'

      dimension psi(*),pot(*),excp(*),cw_sor(*),
     &     b(*),d(*),e(*),f0(*),f1(*),f2(*),f4(*),wgt2(*),
     &     lhs(*),rhs(*),wk0(*),wk1(*),wk2(*),wk3(*),wk4(*),
     &     wk5(*),wk6(*),wk7(*),wk8(*),wk9(*),wk10(*),wk11(*)

c     prepare right and left-hand side of the Poisson equation
c     include the diagonal part of the differentiation operator in lhs

      if (islat.ne.0) then
c        DFT exchange approximation is used
         call focksl(iorb,psi,pot,excp,e,f0,f1,f2,f4,lhs,rhs,
     &        wk0,wk1,wk2,wk3,wk4,wk5,wk6,wk7,wk8,wk9,wk10,wk11)
      else
            call fock (iorb,psi,pot,excp,e,f0,f1,f2,f4,wk0,wk1,wk2)
      endif

      call dcopy (mxsize,foc1,1,lhs,1)
      call dcopy (mxsize,foc2,1,rhs,1)

      ipex=mgx(6,iorb)
      ipex=ipex-2*(ipex/2)
      if (ipex.eq.0) then
         isym= 1 
      else
         isym=-1
      endif

c     start the sor iterations
c     fill wk2 array

      iborb=i1b(iorb)

      do itr1=1,maxsor1	  
         do ig=1,i1ng(iorb)
            muoffs=0
            omega=ovforb(ig)
            omega1=1.d0-omega
            ioffst=ioffs(ig)	  	  
            ioffs1=ioffst+1	  
            iborb1=iborb+ioffst
           
            do i=1,4
               dmu2t(i)=dmu2(i,ig)
               dmu1t(i)=dmu1(i,ig)
            enddo

            do i=1,ngsize(ig)
               lhs(ioffst+i)=foc1(ioffst+i)+diag(ig)
            enddo	    

c            write(*,'(4d25.15)') (lhs(i),i=1,ngsize(ig),20)
            if (i1ng(iorb).eq.1) then 
c              icase=1
               ifill=1
               ngrd1 =ingr1(1,ig)
               ngrd6a=ingr1(2,ig)
               ngrd6b=ingr1(3,ig)
               ngrd7 =ingr1(4,ig)	  	  

c              initialize aorb array needed to evaluate orbital values in
c              the asymptotic region

               nmut=i1mu(iorb)
               do i=nmut-4,nmut
                  ii=(i-1)*nni
                  do j=1,nni
                     ij=ii+j
                     if (f1(ij).eq.0.d0) then
                        wk1(ij)=0.d0
                     else
                        wk1(ij)=lhs(ij)/f1(ij)
                     endif
                  enddo
               enddo
               call iniaorb (nmut,wk0,wk1)
               itail(1)=iorb
               call asymorbtail (psi(iborb),wk0)				
               
               call fill (nni,nmu(ig),isym,psi(iborb),wk2)

               do itr2=1,maxsor(iorb)	  		
		  call sor
     &                 (wk2,lhs(ioffs1),rhs(ioffs1),b(ioffs1),d(ioffs1),
     &                 cw_sor(iadext(ig)),cw_sor(iadnor(ig)),
     &                 cw_sor(iadex1(ig)),cw_sor(iadex2(ig)),
     &                 cw_sor(iadex3(ig)))
               enddo     
               call refill (nni,nmu(ig),psi(iborb),wk2)     
c               call asymorb (nmut,psi(iborb),wk0)				
            else
               if (ig.eq.1) then 
c                 icase=2
                  ifill=2
                  ngrd1 =ingr2(1,ig)
                  ngrd6a=ingr2(2,ig)
                  ngrd6b=ingr2(3,ig)
                  ngrd7 =ingr2(4,ig)	  	  	    
                  call fill2 (nni,nmu(ig),psi(iborb),wk2)
                  do itr2=1,maxsor(iorb)
                     call sor 
     &                    (wk2,lhs(ioffs1),rhs(ioffs1),b(ioffs1),
     &                    d(ioffs1),
     &                    cw_sor(iadext(ig+ngrids)),
     &                    cw_sor(iadnor(ig+ngrids)),
     &                    cw_sor(iadex1(ig+ngrids)),
     &                    cw_sor(iadex2(ig+ngrids)),
     &                    cw_sor(iadex3(ig+ngrids)))
                  enddo     
                  call refill (nni,nmu(ig),psi(iborb),wk2)     
               elseif (ig.ne.1.and.ig.ne.i1ng(iorb)) then 
c                 icase=2
                  ifill=3
                  ngrd1 =ingr2(1,ig)
                  ngrd6a=ingr2(2,ig)
                  ngrd6b=ingr2(3,ig)
                  ngrd7 =ingr2(4,ig)	  	  	    
                  
                  muoffs=iemu(ig-1)-1
                  call fill3 (nni,nmu(ig),psi(iborb),wk2)
                  do itr2=1,maxsor(iorb)	  		
                     call sor 
     &                    (wk2,lhs(ioffs1),rhs(ioffs1),b(ioffs1),
     &                    d(ioffs1),
     &                    cw_sor(iadext(ig+ngrids)),
     &                    cw_sor(iadnor(ig+ngrids)),
     &                    cw_sor(iadex1(ig+ngrids)),
     &                    cw_sor(iadex2(ig+ngrids)),
     &                    cw_sor(iadex3(ig+ngrids)))
                  enddo     
                  call refill34 (nni,nmu(ig),psi(iborb),wk2)     
                  muoffs=0
               elseif (ig.eq.i1ng(iorb)) then 
                  
c                 initialize aorb array needed to evaluate orbital values in
c                 the asymptotic region
                  
                  nmut=i1mu(iorb)
                  do i=nmut-4,nmut
                     ii=(i-1)*nni
                     do j=1,nni
                        ij=ii+j
                        if (f1(ij).eq.0.d0) then
                           wk1(ij)=0.d0
                        else
                           wk1(ij)=lhs(ij)/f1(ij)
                        endif
                     enddo
                  enddo
                  call iniaorb (nmut,wk0,wk1)
                  
                  ifill=4
                  ngrd1 =ingr1(1,ig)
                  ngrd6a=ingr1(2,ig)
                  ngrd6b=ingr1(3,ig)
                  ngrd7 =ingr1(4,ig)	  	  	    
                  
                  muoffs=iemu(ig-1)-1
                  stop
                  call fill4 (nni,nmu(ig),psi(iborb),wk2)
                  do itr2=1,maxsor(iorb)  		
                     call sor 
     &                    (wk2,lhs(ioffs1),rhs(ioffs1),b(ioffs1),
     &                    d(ioffs1),
     &                    cw_sor(iadext(ig)),
     &                    cw_sor(iadnor(ig)),
     &                    cw_sor(iadex1(ig)),
     &                    cw_sor(iadex2(ig)),
     &                    cw_sor(iadex3(ig)))
                  enddo
                  call refill34 (nni,nmu(ig),psi(iborb),wk2)     
                  call asymorb (nmut,psi(iborb),wk0)
                  muoffs=0	    
               endif
            endif
         enddo
         
c        if iortho is nonzero the strong orthogonality conditions are imposed
c        i.e. the orthogonalization is performed every maxsor(iorb) iterations
         
C        if (iortho.eq.1) then
C        iprt=0
C        if (iorb.gt.1) call ortho (iorb,psi,f4,wgt2,wk2,iprt)
C        endif
      enddo

      return
      end
