********************************************************************************
*                                                                              * 
*  2DHF version 2-2006                                                         *
*  Copyright (C) 2006 Jacek Kobus                                              *
*                                                                              *
*  This software may be used and distributed according to the terms            *
*  of the GNU General Public License, see README and COPYING.                  *
*                                                                              *
********************************************************************************
c ### scmc ###

c     Calculates self-consistent multiplicative constant (see eq. 3 in
c     Karasiev, Ludenia, eq.3 PR 64 (2002) 062510)

      subroutine scmc (psi,pot,excp,e,f0,wgt1,wgt2,
     &     wk0,wk1,wk2,wk3,wk4,wk5,wk6,wk7,wk8,wk9,wk10,wk11,wk12,wk13)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      parameter (const13=1.d0/3.d0,const43=4.d0/3.d0)
      include 'commons8.inc'
      dimension psi(*),pot(*),excp(*),e(*),f0(*),wgt1(*),wgt2(*),
     &     wk0(*),wk1(*),wk2(*),wk3(*),wk4(*),wk5(*),wk6(*),
     &     wk7(*),wk8(*),wk9(*),wk10(*),wk11(*),wk12(*),wk13(*)
      data eps /1.d-7/

      if (nel.lt.2) return

c FIXME
c     contribution from coulomb interaction within the same shell

      wdcoul=0.d0

      do iorb=1,norb      
         iborb=i1b(iorb)
         isiorb=i1si(iorb)
         nmut=i1mu(iorb)
         ibpot=i2b(iorb)	  
         isipot=i2si(iorb)
         oc=occ(iorb)

         if (oc.gt.1.d0) then
            call dcopy (isipot,pot(ibpot),1,wk2,1)
            call prod  (isiorb,psi(iborb),wk2)
            call prod  (isiorb,psi(iborb),wk2)
            
            w=ddot(isiorb,wgt2,1,wk2,1)
c            wdcoul=wdcoul+oc*(oc-1.d0)/2.d0*w
            wdcoul=wdcoul+oc/2.d0*w
         endif	    
      enddo

c     contribution from coulomb and exchange interaction between shells
      
      wex1 =0.d0
      wex2 =0.d0

      do iorb1=1,norb
         
c        retrieve from a disk file exchange potentials involving iorb1
         
         if (iform.eq.0.or.iform.eq.2) then
            call  rfdexch(iorb1,excp)
         endif
         
         iborb1=i1b(iorb1)
         isiorb1=i1si(iorb1)
         nmut1=i1mu(iorb1)
         ibpot1=i2b(iorb1)
         isipot1=i2si(iorb1)	  	  
         oc1=occ(iorb1)	
         
         ipe1=mgx(6,iorb1)
         iex1 =iorb1+iorb1*(iorb1-1)/2

         call dcopy (isiorb1,psi(iborb1),1,wk0,1)
         call prod  (isiorb1,psi(iborb1),wk0)

c        calculate exchange interaction within pi, delta, etc. open shell
         
         if (ipe1.gt.0.and.abs(oc1-1.d0).gt.eps) then
            call exint (iorb1,ocx1)
            ibex=i3b(iex1)
            isiex1=i3si(iex1)
            
            ngrid=min(isiorb1,isiex1)
            call prod2  (ngrid,wk0,excp(ibex),wk1)
            w=ddot (ngrid,wgt2,1,wk1,1)
            wex1=wex1+ocx1*w
            
            if (idbg(70).ne.0) then
               write(*,6020) iorb1,w
 6020          format(' ','orb v(x-within shell) ',i4,2x,4e15.5)
            endif
            
            if (idbg(71).ne.0) then
               write(*,7020) iorn(iorb1),bond(iorb1),gut(iorb1),
     &              ipe1,ocx1
 7020          format(' ','v(x-within shell)  ',
     &              i4,1x,a8,a1,i4,f6.2)
            endif
         endif
         
         do iorb2=iorb1+1,norb
            iborb2=i1b(iorb2)
            isiorb2=i1si(iorb2)
            nmut2=i1mu(iorb2)
            ibpot2=i2b(iorb2)
            isipot2=i2si(iorb2)	  
            oc2=occ(iorb2)	

            ipe2=mgx(6,iorb2)
            iex=iorb1+iorb2*(iorb2-1)/2
            ibex=i3b(iex)	  
            isiex=i3si(iex)

c           exchange interaction between shells (same lambda)	  

            call excont (iorb1,iorb2,ocx1,ocx2)	  
            
            ngrid=min(isiorb1,isiorb2,isiex)
            call prod2 (ngrid,excp(ibex),psi(iborb1),wk1)
            call prod  (ngrid,psi(iborb2),wk1)
            
            w=ddot(ngrid,wgt2,1,wk1,1)
            wex1=wex1+ocx1*w
            
c           exchange interaction between shells (different lambda)
            
            if (ilc(iex).gt.1) then
               call prod2 (ngrid,excp(ibex+ngrid),psi(iborb1),wk1)
               call prod  (ngrid,psi(iborb2),wk1)
               w=ddot (ngrid,wgt2,1,wk1,1)
               wex2=wex2+ocx2*w
            endif
 20         continue
         enddo
      enddo

c FIXME
      ehfex=wdcoul+wex1+wex2

      alphaf=two/three
c     exchange energy from DFT functionals 
      call dftex (psi,pot,wgt2,wk0,wk1,wk2,wk3,
     &     wk4,wk5,wk6,wk7,wk8,wk9,wk10,wk11,wk12,wk13)
         
      alphaf=abs(ehfex/edftex)*two/three

c FIXME
      if (idbg(78).ne.0) then
         write(*,'("scmc: ehfex,edftex,alphaf",6e15.6)') 
     &        ehfex,edftex,alphaf
      endif

      return
      end


