********************************************************************************
*                                                                              * 
*  2DHF version 2-2006                                                         *
*  Copyright (C) 1996 Jacek Kobus, Leif Laaksonen, Dage Sundholm               *
*                                                                              *
*  This software may be used and distributed according to the terms            *
*  of the GNU General Public License, see README and COPYING.                  *
*                                                                              *
********************************************************************************
c ### totensl ###

c     Calculates total energy using several DFT functionals

      subroutine totensl (psi,pot,excp,e,f0,wgt1,wgt2,
     &     wk0,wk1,wk2,wk3,rhot,rhotup,rhotdown,
     &     grhot,grhotup,grhotdown,wk10,wk11,wk12,wk13)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
c      integer*4 rhot,rhotup,rhotdown,grhot,grhotup,grhotdown,wk4,wk5,
c     &     wk6,wk7
      parameter (const13=1.d0/3.d0,const43=4.d0/3.d0)
      include 'commons8.inc'	

      dimension psi(*),pot(*),excp(*),e(*),f0(*),wgt1(*),wgt2(*),
     &     wk0(*),wk1(*),wk2(*),wk3(*),rhot(*),rhotup(*),rhotdown(*),
     &     grhot(*),grhotup(*),grhotdown(*),
     &     wk10(*),wk11(*),wk12(*),wk13(*)

c FIXME
      if (idbg(500).ne.0) then
c        test n2f and nfnf routines
         print *,'Testing n2f'
         call testn2f(rhot,wk0,wk1,wk2,wk3)

         print *,''
         print *,'Testing nfng'
         call testnfng(rhot,rhotup,rhotdown,grhotdown,grhotup,
     &        wk0,wk1,wk2,wk3,wk10)

         call n2f(psi,wk0,wk1,wk2,wk3)
         call prod(mxsize,psi,wk3)
      endif


      if (nel.lt.2) return

c     calculate first contributions from one particle operators and
c     Coulomb potential contributions within the same shell

c FIXME
      if (idbg(70).ne.0) then
	 write(*,*) 'orb        t        v(n)      t+v(n)     ',
     &        ' v(c-within shell)'
      endif

      norb2=norb*norb

      vnt=0.d0
      vkt=0.d0
      woneel=0.d0
      wndc =0.d0

      do iorb=1,norb
         iborb=i1b(iorb)
         isiorb=i1si(iorb)
         nmut=i1mu(iorb)
         ibpot=i2b(iorb)	  
         isipot=i2si(iorb)
         oc=occ(iorb)
         
         ipe=mgx(6,iorb)
         ipex=ipe-2*(ipe/2)

c        calculate derivatives over mu and ni variables by means of matrix
c        multiplication

         if (ipex.eq.0) then
	    isym= 1
         else
	    isym=-1
         endif

         call fill  (nni,nmut,isym,psi(iborb),wk3)
         call difni (nmut,wk3,wk0,wk1,wk2)
         call refill(nni,nmut,wk1,wk0)
         call difmu (nmut,wk3,wk2)
         call refill(nni,nmut,wk0,wk2)

c        add derivatives over mu and ni

         call add (isiorb,wk0,wk1)

c        add contribution from phi part of laplasian
c        e enters the expression with minus sign which is already
c        incorporated in e

         if (ipe.ne.0) then
	    w=dble(ipe*ipe)
	    call dcopy (isiorb,e,1,wk0,1)
	    if (ipe.ne.1) then
               call dscal (isiorb,w,wk0,1)
	    endif
	    call prod  (isiorb,psi(iborb),wk0)
	    call add   (isiorb,wk0,wk1)
         endif
         
         call dcopy (isiorb,wk1,1,wk2,1)
         call prod  (isiorb,psi(iborb),wk2)
         w=ddot(isiorb,wgt1,1,wk2,1)
         vk(iorb)=w
         
         call dcopy (isiorb,f0,1,wk0,1)
         call prod  (isiorb,psi(iborb),wk0)
         
         call prod2 (isiorb,psi(iborb),wk0,wk2)
         w=ddot(isiorb,wgt1,1,wk2,1)
         vn(iorb)=w

         call add	 (isiorb,wk0,wk1)
         call prod (isiorb,psi(iborb),wk1)
         
         w = ddot(isiorb,wgt1,1,wk1,1)
         woneel=woneel+oc*w

         vnt=vnt+oc*vn(iorb)
         vkt=vkt+oc*vk(iorb)
      enddo

c     contribution from coulomb interaction within the same shell

      if (isipot.ne.isiorb) then
         write(iout6,*) 'coulomb potentials and orbitals have to ',
     &        'defined on the same number of subgrids'
         stop 'totensl'
      endif				

c     calculate the coulomb potential contribution from all orbitals
c     (include 1/2 factor )

      do i=1,mxsize
         wk2(i)=0.d0
      enddo
      
      do iorb=1,norb
         ibpot=i2b(iorb)
         isipot=i2si(iorb)	
         oc=occ(iorb)/two
         call daxpy (isipot,oc,pot(ibpot),1,wk2,1)
      enddo

c     contribution from the Coulomb interaction

      do iorb=1,norb
         iborb=i1b(iorb)
         isiorb=i1si(iorb)	  
         call prod2 (isiorb,psi(iborb),psi(iborb),wk0)
         call prod (isiorb,wk2,wk0)
         call dscal (isiorb,occ(iorb),wk0,1)
         w=ddot(isiorb,wgt2,1,wk0,1)
         wndc=wndc+w
      enddo
      
c     DFT exchange energy corrections

      wex=0.d0
      wcorr=0.d0

      if     (idftex.eq.1) then
         wex=exxalpha(psi,wgt2,rhot,rhotup,rhotdown,
     &           grhot,grhotup,grhotdown,
     &           wk0,wk1,wk2,wk3,wk10,wk11,wk12,wk13)
      elseif (idftex.eq.2) then
         wex=exbe88(psi,wgt2,rhot,rhotup,rhotdown,
     &        grhot,grhotup,grhotdown,
     &        wk0,wk1,wk2,wk3,wk10,wk11,wk12,wk13)
      elseif (idftex.eq.3) then
         wex=expw86(psi,wgt2,rhot,rhotup,rhotdown,
     &        grhot,grhotup,grhotdown,
     &        wk0,wk1,wk2,wk3,wk10,wk11,wk12,wk13)
      elseif (idftex.eq.4) then
         wex=expw91(psi,wgt2,rhot,rhotup,rhotdown,
     &        grhot,grhotup,grhotdown,
     &        wk0,wk1,wk2,wk3,wk10,wk11,wk12,wk13)
      endif

c     DFT correlation energy corrections

      if (idftcorr.eq.1) then
         wcorr=eclyptot(psi,wgt2,rhot,rhotup,rhotdown,
     &           grhot,grhotup,grhotdown,
     &           wk0,wk1,wk2,wk3,wk10,wk11,wk12,wk13)
      elseif (idftcorr.eq.2) then
         wcorr=ecvwntot(psi,wgt2,rhot,rhotup,rhotdown,
     &           grhot,grhotup,grhotdown,
     &           wk0,wk1,wk2,wk3,wk10,wk11,wk12,wk13)

      endif

      evt=woneel+wndc+wex+wcorr

c FIXME
      engt(1)=evt
      etot=evt+z1*z2/r
      virrat=evt/vkt

      enkin=vkt
      ennucel=vnt
      encoul=wndc
      enexch=wex

      encouldft=wndc
      enexchdft=wex

      edftcorr=wcorr

      entot=enkin+ennucel+encoul+enexch+edftcorr+z1*z2/r

      if (idbg(70).ne.0) then
         write(*,*)'totesl - woneel,wndc,wex,etotal ',
     &        woneel,wndc,wex,evt
      endif

      return
      end


