***************************************************************************
*                                                                         *
*   Copyright (C) 1996 Leif Laaksonen, Dage Sundholm                      *
*   Copyright (C) 1996-2008 Jacek Kobus <jkob@fizyka.umk.pl>              *
*                                                                         *     
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License version 2 as     *
*   published by the Free Software Foundation.                            *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************
c ### x2dhf ###
c
c     This is the main routine of the 2DHF program.
c     Dimensions of working arrays have to be defined either statically 
c     or dynamically, i.e. based on input data (see prep_src).
                                                                      
      PROGRAM  x2dhf
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)

c     FIXME the following variables (used as pointers) must be 4-byte long
c     when 32-bit machine and 64-bit integers are used

      integer*4 cw_orb_add(1),cw_coul_add(1),cw_exch_add(1),
     &		cw_suppl_add(1),cw_sctch_add(1),cw_sor_add(1), 
     &		i4,i8,offset1,offset2,offset3,offset4,
     &		offset5,offset6,size
      include 'commons8.inc'

      data i4,i8/4,8/

c     set number of additional arrays
      nsuppl=14
      nsctch=14

c     no and nons denote the number of all the orbitals and number 
c     of non sigma type orbitals, respectively. ni and mu correspond to
c     the number of grid points in ni and mu variables.

100   continue

c     read input data and echo input parameters
c     open input/output files

      call rinputd(iform,ni,mu,no,nons)

      ni8=ni+8
      mu8=mu+8
      ngrid=ni*mu
      ngrid8    = ni8*mu8

      length1   = no*ngrid
      length2   = no*ngrid

      if     (iform.eq.1.or.iform.eq.3) then
         length3=( no*(no+1)/2+nons*(nons+1)/2 )*ngrid
      elseif (iform.eq.0.or.iform.eq.2) then
         length3 = (no+nons)*ngrid	 
      endif

c     if HFS or OED method of calculation is chosen set length3 to
      
      if (imethod.eq.2) length3 = ngrid	 

      if (imethod.eq.3.and.iscmc.eq.0) length3 = ngrid	 
      

      length4   = nsuppl*ngrid
      length5   = nsctch*ngrid8     

      length6   = 2*ngrid8+4*ni*(mu+16)+2*maxgrids*(ni+mu)
      lexchrecl = ni*mu*8

c     dynamic allocation of memory acccording to the current case 


      size=lengthfp

c      length=8*length1
      call memalloc (cw_orb_add,  length1, size, offset1)

c      length=8*length2
      call memalloc (cw_coul_add, length2, size, offset2)
	
c      length=8*length3
      call memalloc (cw_exch_add, length3, size, offset3)

c      length=8*length4
      call memalloc (cw_suppl_add,length4, size, offset4)

c      length=8*length5
      call memalloc (cw_sctch_add,length5, size, offset5)

c      length=8*length6

c     for 32-bit indexes allocation for cw_sor could be halved by
c     changing i8 into i4 
      call memalloc (cw_sor_add,  length6, i8, offset6)

c     zeroise arrays

      call zeroarray (cw_orb_add(offset1),cw_coul_add(offset2),
     &     cw_exch_add(offset3),cw_suppl_add(offset4),
     &     cw_sctch_add(offset5),cw_sor_add(offset6))


c     initialize arrays within common blocks, 
c     initialize arrays of variable length and check input data
c     prepare an environment for poisson's equation solving routines
c     prepare arrays determining ordering of mesh points

      call inisuppl (cw_suppl_add(offset4),cw_sor_add(offset6))
c     print out data defining the case

      call printall

c     initialize orbitals, Coulomb and exchange potentials

      call inifun (cw_orb_add(offset1),cw_coul_add(offset2),
     &     cw_exch_add(offset3),cw_suppl_add(offset4),
     &     cw_sctch_add(offset5))

c  perform scf

      call  scf (cw_sor_add(offset6),cw_orb_add(offset1),
     &     cw_coul_add(offset2),cw_exch_add(offset3),
     &     cw_suppl_add(offset4),cw_sctch_add(offset5))

      if (icont.gt.0) goto 100 

      call memdealloc (cw_orb_add,  length1, size, offset1)
      call memdealloc (cw_coul_add, length2, size, offset2)
      call memdealloc (cw_exch_add, length3, size, offset3)
      call memdealloc (cw_suppl_add,length4, size, offset4)
      call memdealloc (cw_sctch_add,length5, size, offset5)
      call memdealloc (cw_sor_add,  length6, i8, offset6)

      stop
      end
      
      include 'blk-data.inc'
